program SimulatorSklopke; { RTK 1988.2.3 }
uses Crt;
type 
  CasT = integer; 
  OsT = 1..2; 

  function Cas: CasT; forward; 
  function Oznaka(Os: OsT): boolean; forward; 
  procedure Pogon(Pospesek1: real); forward; 
  procedure Sklopi; forward; 

{ v1 in v2 bi morali biti pravzaprav lokalni spremenljivki podprogramov
  Sklopka in Sklopka2, vendar ju imamo deklarirani e tule, da ju bo
  lahko podprogram PrikaziStanje izpisoval na zaslon. }
var v1, v2: real; { ocenjeni hitrosti osi, v rnih pasovih na sekundo }

procedure Sklopka;  
var 
  Uvod1, Uvod2: integer;   { koraki, potrebni za zaetek meritve } 
  Ozn1, Ozn2, OznPrej1, OznPrej2: boolean; { za iskanje zaetka oznake } 
  t, t1, t2: CasT;         { as zadnje oznake } 
  dt1, dt2: CasT;          { as med dvema oznakama } 
  RelNapaka: real;         { relativna razlika hitrosti osi } 
  Izenaceno: boolean;      { hitrosti osi v okviru napake } 
begin {Sklopka} 
  Uvod1 := 2; Uvod2 := 2; Izenaceno := false; 
  OznPrej1 := Oznaka(1); OznPrej2 := Oznaka(2); t1 := 0; t2 := 0; 
  repeat 
    t := Cas; Ozn1 := Oznaka(1); Ozn2 := Oznaka(2); 
    if Ozn1 and not OznPrej1 then begin 
      dt1 := t - t1; t1 := t; if Uvod1 > 0 then Uvod1 := Uvod1 - 1; 
    end; {if}
    if Ozn2 and not OznPrej2 then begin 
      dt2 := t - t2; t2 := t; if Uvod2 > 0 then Uvod2 := Uvod2 - 1; 
    end; {if}
    OznPrej1 := Ozn1; OznPrej2 := Ozn2; 
    if (Uvod1 = 0) and (Uvod2 = 0) then begin 
      v1 := 1000.0 / dt1; v2 := 1000.0 / dt2;
      RelNapaka := (v1 - v2) / v2; 
      Pogon(-RelNapaka); 
      Izenaceno := Abs(RelNapaka) < 0.01; 
    end; {if}
  until Izenaceno; 
  Sklopi; 
end; {Sklopka} 

{ Tale razliica skua biti bolj natanna pri merjenju asa
  (kar je lahko koristno, e se osi vrtita prehitro ali pa so
  rni in beli pasovi preozki).  Namesto merjenja asa od zaetka 
  ene rne oznake do zaetka naslednje gleda raje po ve oznak 
  skupaj.  ase, v katerih je prvi opazila posamezno oznako, 
  hrani v vrsti (po ena vrsta za vsako os).  Hitrost oceni prek 
  obdobja vsaj MinDt milisekund, tako da napaka zaradi 
  nenatannosti pri merjenju asa ne bi smela biti prevelika.  
  Seveda pa, e je MinDt prevelik oz. se hitrost vrtenja spreminja 
  prehitro, bo ta izraun spet nenatanen, ker bo izraunal
  povpreno hitrost prek obdobja MinDt milisekund, ne pa trenutne
  hitrosti. }
procedure Sklopka2;  
const 
  MaxN = 100;  { najveje t. meritev v vrsti }
  MinDt = 100; { ms }
var 
  Uvod1, Uvod2: integer;   { koraki, potrebni za zaetek meritve } 
  Ozn1, Ozn2, OznPrej1, OznPrej2: boolean; { za iskanje zaetka oznake } 
  t: CasT;
  t1, t2: array[0..MaxN - 1] of integer; { vrsti }
  i1, i2: integer;         { indeks najstarejega asa v vrsti }
  n1, n2: integer;         { tevilo asov v vrsti }
  RelNapaka: real;         { relativna razlika hitrosti osi } 
  Izenaceno: boolean;      { hitrosti osi v okviru napake } 
begin {Sklopka} 
  Uvod1 := 2; Uvod2 := 2; Izenaceno := false; 
  OznPrej1 := Oznaka(1); OznPrej2 := Oznaka(2); 
  i1 := 0; i2 := 0; n1 := 0; n2 := 0; v1 := 0; v2 := 0;
  repeat 
    t := Cas; Ozn1 := Oznaka(1); Ozn2 := Oznaka(2); 
    if Ozn1 and not OznPrej1 then begin 
      if Uvod1 > 0 then Uvod1 := Uvod1 - 1; 
      if (n1 = MaxN) or ((Uvod1 = 0) and (t - t1[i1] >= MinDt)) then 
        begin v1 := 1000.0 * n1 / (t - t1[i1]); i1 := (i1 + 1) mod MaxN; n1 := n1 - 1; end;
      t1[(i1 + n1) mod MaxN] := t; n1 := n1 + 1;
    end; {if}
    if Ozn2 and not OznPrej2 then begin 
      if Uvod2 > 0 then Uvod2 := Uvod2 - 1; 
      if (n2 = MaxN) or ((Uvod2 = 0) and (t - t2[i2] >= MinDt)) then 
        begin v2 := 1000.0 * n2 / (t - t2[i2]); i2 := (i2 + 1) mod MaxN; n2 := n2 - 1; end;
      t2[(i2 + n2) mod MaxN] := t; n2 := n2 + 1;
    end; {if}
    OznPrej1 := Ozn1; OznPrej2 := Ozn2; 
    if (v1 > 0) and (v2 > 0) then begin 
      RelNapaka := (v1 - v2) / v2; 
      Pogon(-RelNapaka); 
      Izenaceno := Abs(RelNapaka) < 0.01; 
    end; {if}
  until Izenaceno; 
  Sklopi; 
end; {Sklopka} 

{-----------------------------------------------------------------------------}
{ Okolje za testiranje zgornjih reitev.                                      }
{-----------------------------------------------------------------------------}

const StOznak = 4; { po toliko oznak vsake barve }
var 
  CasMs: CasT;
  { Kot je v stopinjah, hitrost je v obratih na sekundo, 
    pospeek pa v obratih na kvadratno sekundo. }
  Kot, Hitrost, Pospesek: array[OsT] of real;

procedure PrikaziStanje;
const ImeOznake: array[boolean] of string = (' ', '#');
begin
  Write('T=', (0.001 * CasMs):6:3, ' s, ',
        'Os1:[', ImeOznake[Oznaka(1)], ' fi=', Kot[1]:3:0, ' w=', Hitrost[1]:5:2, ' w''=', (v1 / StOznak):5:2, ' e=', Abs(Hitrost[1] - v1 / StOznak):0:2, ' a=', Pospesek[1]:5:2, '] ',
        'Os2:[', ImeOznake[Oznaka(2)], ' fi=', Kot[2]:3:0, ' w=', Hitrost[2]:5:2, ' w''=', (v2 / StOznak):5:2, ' e=', Abs(Hitrost[2] - v2 / StOznak):0:2, ' a=', Pospesek[2]:5:2, ']'#13);
end; {PrikaziStanje}

procedure KorakNaprej;
const DeltaMs = 1;
var Os: OsT; c: char;
begin

  { Poglejmo, e je uporabnik pritisnil kakno tipko. }
  if KeyPressed then begin
    c := UpCase(ReadKey); if c = #0 then ReadKey;
    if c = 'P' then begin 
      Write('(Pavza.)', #13);
      c := UpCase(ReadKey); if c = #0 then ReadKey;
      PrikaziStanje;
    end else if (c = 'X') or (c = 'Q') or (c = #27) then
      Halt;
  end; {if}

  { Premaknimo se za 1 ms naprej.  Popravimo kota in hitrosti. }
  CasMs := CasMs + DeltaMs;
  for Os := 1 to 2 do begin
    Kot[Os] := Kot[Os] + 0.001 * DeltaMs * 360 * Hitrost[Os];
    while Kot[Os] >= 360 do Kot[Os] := Kot[Os] - 360;
    Hitrost[Os] := Hitrost[Os] + 0.001 * DeltaMs * Pospesek[Os];
    if Hitrost[Os] < 0.1 then Hitrost[Os] := 0.1;
    { Enkrat na sekundo tudi spremenimo pospeek druge osi. }
    if (CasMs mod 1000) = 0 then Pospesek[Os] := 0.1 * (Random(21) - 10);
  end; {for Os}
  { Obasno prikaimo novo stanje na zaslon. }
  if (CasMs mod 10) = 0 then PrikaziStanje;
  
end; {KorakNaprej}

function Cas: CasT; 
begin
  KorakNaprej;
  Cas := CasMs;
end; {Cas}

function Oznaka(Os: OsT): boolean; 
const DveOznaki = 360 div StOznak; { toliko stopinj pokrivata skupaj en rn in en bel pas }
var i: integer;
begin
  i := Trunc(Kot[Os]) mod DveOznaki;
  Oznaka := i < DveOznaki div 2;
end; {Oznaka}

procedure Pogon(Pospesek1: real); 
begin
  Pospesek[1] := Pospesek1;
end; {Pogon}

procedure Sklopi; 
begin
  PrikaziStanje;
  WriteLn;
  WriteLn('Sklop pri hitrostih ', Hitrost[1]:0:2, ' in ', Hitrost[2]:0:2, ' (razlika: ',
          Abs(Hitrost[1] - Hitrost[2]):0:2, ').');
end; {Sklopi}

begin {SimulatorSklopke}
  WriteLn('Simulator sklopke');
  WriteLn;
  WriteLn('# = pove, e je pod senzorjem rn pas');
  WriteLn('fi = poloaj ploe');
  WriteLn('w = hitrost (v vrtljajih na sekundo),');
  WriteLn('w'' = hitrost, kot jo je ocenil podprogram Sklopka,');
  WriteLn('a = pospeek (v vrtljajih na kvadratno sekundo).');
  WriteLn;
  WriteLn('Pritisni P za pavzo, Q ali X za izhod iz programa.');
  WriteLn;

  Kot[1] := Random(360); 
  Hitrost[1] := 0.1 * (Random(50) + 30);
  Pospesek[1] := 0;
  Kot[2] := Random(360);
  Hitrost[2] := 0.1 * (Random(50) + 30);
  Pospesek[2] := 0.1 * (Random(21) - 10);
  CasMs := 0;
  v1 := 0; v2 := 0; 
  PrikaziStanje;
  
  Sklopka;
end. {SimulatorSklopke}
