program GradnjaLabirintov; { RTK 1989.2.4 }

{ Kot parameter na ukazni vrstici lahko podamo
  tevilko algoritma: 1 = iskanje v globino,
  2 = Prim, 3 = Kruskal. }

uses Crt;
const XS = 77; YS = 21; 
type Celica = (Zid, Pot); 
var Tabela: array[0..XS - 1, 0..YS - 1] of Celica; 

function JeZid(x, y: integer): boolean; 
begin
  if (x < 0) or (x >= XS) or (y < 0) or (y >= YS) then JeZid  := false
  else JeZid := Tabela[x, y] = Zid; 
end; {JeZid} 

procedure Porusi(x, y: integer);
var ch: char;
begin
  Tabela[x, y] := Pot;
  GotoXY(x + 1, y + 1); Write('.');
  if KeyPressed then begin
    ch := UpCase(ReadKey); if ch = #0 then ReadKey;
    if ch = 'P' then begin
      ch := UpCase(ReadKey); if ch = #0 then ReadKey;
    end else if (ch = 'X') or (ch = 'Q') or (ch = #27) then
      Halt;
  end; {if}
  Delay(30);
end; {Porusi}

{-----------------------------------------------------------------------------}

procedure Labirint(x, y: integer); 
var nx, ny: integer; 
begin
  Porusi(x, y); 
  while JeZid(x - 2, y) or JeZid(x + 2, y) or 
    JeZid(x, y - 2) or JeZid(x, y + 2) do begin 
    nx := x; ny := y; 
    case Random(4) of 
      0: nx := x + 2; 
      1: nx := x - 2; 
      2: ny := y + 2; 
      3: ny := y - 2; 
    end; {case} 
    if JeZid(nx, ny) then begin
      Porusi((x + nx) div 2, (y + ny) div 2); 
      Labirint(nx, ny); 
    end; {if}
  end; {while}
end; {Labirint} 

{-----------------------------------------------------------------------------}

procedure LabirintPrim(x0, y0: integer); 
type TockaT = record x, y: integer end;
var ToDo: array[1..XS * YS] of TockaT; nToDo, x, y, nx, ny, i: integer;  
begin
  nToDo := 1; with ToDo[1] do begin x := x0; y := y0 end;
  Porusi(x0, y0); 
  while nToDo > 0 do begin 
    { Nakljuno izberimo kakno izmed tistih celic, ki imajo
      e kakno primerno zazidano sosedo.  Seznam takih celic
      je v tabeli ToDo. }
    i := Random(nToDo) + 1; x := ToDo[i].x; y := ToDo[i].y; 
    if not (JeZid(x - 2, y) or JeZid(x + 2, y) or 
      JeZid(x, y - 2) or JeZid(x, y + 2)) then begin 
      ToDo[i] := ToDo[nToDo]; nToDo := nToDo - 1; 
    end {if}
    { Izberimo eno od zazidanih sosed trenutne celice 
      in tisti zid poruimo.  Sosedo dodajmo na seznam ToDo,
      da bomo o priliki pregledali e njene sosede. }
    else while true do begin 
      nx := x; ny := y; 
      case Random(4) of 
        0: nx := x + 2;
        1: nx := x - 2;
        2: ny := y + 2;
        3: ny := y - 2; 
      end; {case} 
      if JeZid(nx, ny) then begin 
        Porusi((x + nx) div 2, (y + ny) div 2); 
        Porusi(nx, ny); 
        nToDo := nToDo + 1; ToDo[nToDo].x := nx; ToDo[nToDo].y := ny; 
        break; 
      end; {if}
    end; {while}
  end; {while}
end; {LabirintPrim} 

{-----------------------------------------------------------------------------}

procedure LabirintKruskal; 
type TockaT = record x, y: integer end;
var  
  ToDo: array[1..XS * YS] of TockaT; 
  nToDo, x, y, xt, yt, k, nk, nx, ny, i: integer;  
  Komp: array[1..XS, 1..YS] of integer; 
 
  function IstaKomp(x, y, KotKomp: integer): boolean; 
  begin
    if (x < 0) or (x >= XS) or (y < 0) or (y >= YS) then IstaKomp := true
    else IstaKomp := Komp[x, y] = KotKomp; 
  end; {IstaKomp} 

begin
  nToDo := 0; 
  { Labirint bo med gradnjo razdeljen na "komponente" -- dele
    labirinta, ki so z zidovi povsem loeni drug od drugega. 
    Na zaetku je vsaka celica, ki ima lihi koordinati, sama
    svoja komponenta. }
  for x := 0 to (XS div 2) - 1 do for y := 0 to (YS div 2) - 1 do begin 
    Inc(nToDo); Komp[2 * x + 1, 2 * y + 1] := nToDo; 
    ToDo[nToDo].x := 2 * x + 1; ToDo[nToDo].y := 2 * y + 1; 
    Porusi(2 * x + 1, 2 * y + 1); 
  end; {for}
  { Dokler obstaja ve kot ena komponenta: izberimo nakljuno 
    neko toko in neko njeno sosedo; e nista v isti komponenti,
    zid med njima poruimo (obe komponenti se s tem zlijeta 
    v eno samo. }
  while nToDo > 0 do begin 
    i := Random(nToDo) + 1; x := ToDo[i].x; y := ToDo[i].y; k := Komp[x, y]; 
    if  IstaKomp(x - 2, y, k) and IstaKomp(x + 2, y, k) 
    and IstaKomp(x, y - 2, k) and IstaKomp(x, y + 2, k) then begin 
      ToDo[i] := ToDo[nToDo]; nToDo := nToDo - 1; 
    end {if}
    else while true do begin 
      nx := x; ny := y; 
      case Random(4) of 
        0: nx := x + 2;
        1: nx := x - 2;
        2: ny := y + 2;
        3: ny := y - 2; 
      end; {case} 
      if not IstaKomp(nx, ny, k) then begin { poruimo zid med njima }
        Porusi((x + nx) div 2, (y + ny) div 2); nk := Komp[nx, ny]; 
        for xt := 0 to (XS div 2) - 1 do for yt := 0 to (YS div 2) - 1 do 
          if Komp[2 * xt + 1, 2 * yt + 1] = nk then 
            Komp[2 * xt + 1, 2 * yt + 1] := k; 
        break; 
      end; {if}
    end; {while}
  end; {while}
end; {LabirintKruskal} 

{-----------------------------------------------------------------------------}

procedure ZazidajTabelo; 
var x, y: integer; 
begin
  for x := 0 to XS - 1 do 
    for y := 0 to YS - 1 do 
      Tabela[x,y] := Zid; 
end; {ZazidajTabelo} 

procedure PrikaziTabelo; 
var x, y: integer; 
begin
  GotoXY(1, 1);
  for y := 0 to YS - 1 do begin 
    for x := 0 to XS - 1 do 
      if Tabela[x, y] = Zid then Write('#') else Write('.'); 
    WriteLn; 
  end; {for}
end; {PrikaziTabelo} 

var Alg, c, x0, y0: integer;
begin {GradnjaLabirintov}

  if ParamCount > 0 then Val(ParamStr(1), Alg, c)
  else Alg := 1;

  ZazidajTabelo; Randomize;
  ClrScr; PrikaziTabelo; 
  x0 := 2 * Random(XS div 2) + 1; y0 := 2 * Random(YS div 2) + 1;
  if Alg = 1 then Labirint(x0, y0)
  else if Alg = 2 then LabirintPrim(x0, y0)
  else if Alg = 3 then LabirintKruskal;
  PrikaziTabelo;
  
end. {GradnjaLabirintov}
