program SprejemanjePaketov; { RTK 1990.3.3 }
uses Crt;
const 
  MaxDatotek = 10; 
  PovpDolzina = 100;
  MaxPaketov = 3 * MaxDatotek * PovpDolzina; { ve kot tevilo datotek * povprena dolina } 
type 
  ZapisT = integer;
  ShranjenZapisT = record     { oblika podatkov o prispelih zapisih } 
    Stevilka: integer;        { tevilka zapisa v datoteki } 
    Naslednji: integer;       { naslednji element v verigi } 
    Zapis: ZapisT;            { vsebina zapisa } 
  end; {ShranjenZapisT} 
  OpisDatotekeT = record      { oblika opisa prihajajoe datoteke } 
    ZadnjiZapis: integer;     { tevilka zadnjega zapisa } 
    Sprejetih: integer;       { tevilo sprejetih zapisov } 
    PrviZapis: integer;       { prvi element v verigi zapisov } 
  end; {OpisDatotekeT} 
  PaketT = record
    VrstaPaketa: (Vmesni, Zadnji); { vrsta prihajajoega paketa }
    Stevilka: integer;        { zaporedna tevilka paketa }
    Datoteka: 1..MaxDatotek;  { tevilka datoteke, ki ji paket pripada }
    Zapis: ZapisT;            { zapis v paketu }
  end; {PaketT}
  
var 
  PrazniZapisi: integer; { kae na prvi element v verigi prostih zapisov v tabeli Zapisi } 
  Opisi: array[1..MaxDatotek] of OpisDatotekeT;     { opisi datotek } 
  Zapisi: array[1..MaxPaketov] of ShranjenZapisT;   { seznam zapisov } 
  StPraznih: integer;                               { le zaradi statistike }
 
  procedure Pisi(Datoteka: integer; Zapis: ZapisT); forward;

{ Pripravi tabeli Opisi in Zapisi. } 
procedure PripraviPakete; 
var i: integer; 
begin
  { Cela tabela Zapisi je ena sama dolga veriga praznih zapisov. } 
  PrazniZapisi := 1; StPraznih := MaxPaketov;
  for i := 1 to MaxPaketov - 1 do Zapisi[i].Naslednji := i + 1; 
  Zapisi[MaxPaketov].Naslednji := -1; 
  { Na zaetku e ne sprejemamo nobene datoteke. } 
  for i := 1 to MaxDatotek do with Opisi[i] do 
    begin ZadnjiZapis := 0; Sprejetih := 0; PrviZapis := -1 end;
end; {PripraviPakete} 

{ Sprosti prostor v tabeli Zapisi, ki ga je zasedala datoteka Dat.
  Njene zapise postavimo na zaetek verige praznih zapisov. }
procedure SprostiDatoteko(Dat: integer); 
var i, j: integer; 
begin
  j := PrazniZapisi; 
  with Opisi[Dat] do begin 
    i := PrviZapis; PrviZapis := -1; StPraznih := StPraznih + Sprejetih;
    ZadnjiZapis := 0; Sprejetih := 0; PrazniZapisi := i; 
  end; {with} 
  while Zapisi[i].Naslednji <> -1 do i := Zapisi[i].Naslednji; 
  Zapisi[i].Naslednji := j; 
end; {SprostiDatoteko} 

{ Shrani prispeli paket p v tabelo Zapisi in 
  izpie datoteko, e so prili e vsi zapisi. } 
procedure ShraniPaket(p: PaketT); 
var i, j: integer; 
begin
  if p.VrstaPaketa = Zadnji then 
    Opisi[p.Datoteka].ZadnjiZapis := p.Stevilka; 
  Opisi[p.Datoteka].Sprejetih := Opisi[p.Datoteka].Sprejetih + 1; 
  i := PrazniZapisi; PrazniZapisi := Zapisi[i].Naslednji; 
  StPraznih := StPraznih - 1;
  with Zapisi[i] do begin 
    Naslednji := Opisi[p.Datoteka].PrviZapis; 
    Stevilka := p.Stevilka; Zapis := p.Zapis; 
  end; {with} 
  Opisi[p.Datoteka].PrviZapis := i; 
  with Opisi[p.Datoteka] do 
    if Sprejetih = ZadnjiZapis then begin 
      for i := 1 to ZadnjiZapis do begin 
        j := PrviZapis; 
        while Zapisi[j].Stevilka > i do j := Zapisi[j].Naslednji; 
        Pisi(p.Datoteka, Zapisi[j].Zapis); 
      end; {for}
      SprostiDatoteko(p.Datoteka); 
    end; {if}
end; {ShraniPaket} 

{-----------------------------------------------------------------------------}
{ Preostanek programa simulira prejem paketov od poiljatelja.  Prihajajo za  }
{ ve datotek naenkrat v poljubno premeanem vrstnem redu.                    }
{-----------------------------------------------------------------------------}

type 
  PosiljanjeDatotekeT = record
    Dolzina, Poslanih, Izpisanih: integer;
    Zamrznjena: boolean;
    Zapisi: array[1..MaxPaketov] of ZapisT;
    Poslani: array[1..MaxPaketov] of boolean;
  end; {PosiljanjeDatotekeT}
var
  Datoteke: array[1..MaxDatotek] of PosiljanjeDatotekeT;

{ Nakljuno si izbere eno od datotek, ki jih trenutno 
  poiljamo (in ki niso zamrznjene).  Za to datoteko si
  potem nakljuno izbere enega od e ne poslanih paketov
  in ga polje (poklie podprogram ShraniPaket). }
procedure PosljiNovPaket;
var i, n, Dat: integer; P: PaketT;
begin
  { Koliko datotek se sploh poilja? }
  n := 0;
  for i := 1 to MaxDatotek do with Datoteke[i] do
    if (Dolzina > 0) and not Zamrznjena then n := n + 1;
  if n = 0 then exit;
  { Nakljuno izberimo eno od teh datotek. }
  n := Random(n); Dat := 1; 
  while Dat <= MaxDatotek do with Datoteke[Dat] do begin
    if (Dolzina > 0) and not Zamrznjena then
      if n = 0 then break else n := n - 1;
    Dat := Dat + 1;
  end; {while, with}
  { Izberimo nek zapis, ki ga e nismo poslali. }
  with Datoteke[Dat] do begin
    n := Random(Dolzina - Poslanih); i := 1;
    while i <= Dolzina do 
      if not Poslani[i] then
        begin if n = 0 then break else n := n - 1 end
      else i := i + 1;
    Assert(i <= Dolzina);
    { Ta zapis poljimo v paketu. }
    with P do begin 
      if i = Dolzina then VrstaPaketa := Zadnji
      else VrstaPaketa := Vmesni;
      Stevilka := i;
      Datoteka := Dat;
      Zapis := Zapisi[i];
    end; {with P}
    ShraniPaket(P);
    Poslani[i] := true; Poslanih := Poslanih + 1;
  end; {with Datoteke[Dat]}
end; {PosljiNovPaket}

{ Zane poiljati novo datoteko pod tevilko Dat
  (ki mora biti trenutno prosta).  Dolino in
  vsebino ji izbere sam. }
procedure NovaDatoteka(Dat: integer);
var i: integer;
begin
  with Datoteke[Dat] do begin
    Assert(Dolzina <= 0);
    Dolzina := Random(2 * PovpDolzina - 1) + 1;
    Poslanih := 0; Izpisanih := 0; 
    Zamrznjena := false;
    for i := 1 to Dolzina do 
      begin Zapisi[i] := Random(10000); Poslani[i] := false; end;
  end; {with}
end; {NovaDatoteka}

{ To je podprogram, ki naj bi ga tekmovaleva koda
  klicala za shranjevanje prispelih zapisov. 
  Naa implementacija tega podprograma bo preverila,
  e poskua tekmovaleva koda zapise res shraniti
  v pravem vrstnem redu in s pravo vsebino. }
procedure Pisi(Datoteka: integer; Zapis: ZapisT); 
begin
  with Datoteke[Datoteka] do begin
    Assert(Dolzina > 0);
    Assert(Izpisanih < Dolzina);
    Izpisanih := Izpisanih + 1;
    Assert(Zapis = Zapisi[Izpisanih]);
    { e je bila izpisana e cela datoteka,
      sprostimo svoj zapis Datoteke[Dat]. }
    if Izpisanih = Dolzina then begin
      Assert(Poslanih = Dolzina);
      Dolzina := 0; Izpisanih := 0; 
      Zamrznjena := false;
    end; {if}
  end; {with}
end; {Pisi}

procedure PrikaziStanje;
var i: integer;
begin
  Write('[', StPraznih:4, '/', MaxPaketov:4, '] ');
  for i := 1 to MaxDatotek do with Datoteke[i] do begin
    if Dolzina <= 0 then Write('------- ')
    else Write(Poslanih:3, '/', Dolzina:3, ' ');
  end; {for, with}
  Write(#13);
end; {PrikaziStanje}

var i: integer; c: char;
begin {SprejemanjePaketov}
  WriteLn('Simulator sprejemanja in poiljanja paketov');
  WriteLn('S pritiski na tipke 1..0 se zane poiljati nova datoteka,');
  WriteLn('e pa se datoteka pod tisto tevilko e poilja, se ob');
  WriteLn('pritisku na to tipko poiljanje zaasno zamrzne/odmrzne.');
  WriteLn('P = pavza.  Q ali X = izhod iz programa.');
  WriteLn;
  for i := 1 to MaxDatotek do Datoteke[i].Dolzina := 0;
  PripraviPakete;
  while true do begin
    if KeyPressed then begin
      c := UpCase(ReadKey); if c = #0 then ReadKey;
      if c = 'P' then begin
        Write('(Pavza.)', #13);
        c := ReadKey; if c = #0 then ReadKey;
        PrikaziStanje;
        continue;
      end else if (c = 'Q') or (c = 'X') or (c = #27) then 
        break
      else if c in ['0'..'9'] then begin
        i := Ord(c) - Ord('0'); if i = 0 then i := 10;
        with Datoteke[i] do 
          if Dolzina > 0 then Zamrznjena := not Zamrznjena
          else NovaDatoteka(i);
      end; {if}
    end; {if}
    PosljiNovPaket;
    PrikaziStanje;
    Delay(10);
  end; {while}
end. {SprejemanjePaketov}
