program NajdaljseNarascajocePodzaporedje; { RTK 1992.3.3 }
uses Windows;
const n = 10000;
type Zaporedje = array[1..n] of integer;

{ Reitev s tabelo v[j], ki za vsak j hrani najmanji doslej
  prebrani element tabele A, pri katerem se konuje kakno
  strogo naraajoe zaporedje doline j.  
  asovna zahtevnost: O(n k), e je k dolina najdaljega 
  strogo naraajoega podzaporedja. }
procedure Najdaljse(A: Zaporedje; DolA: integer;
                    var B: Zaporedje; var DolB: integer);
var v, w, Predhodnik: Zaporedje; i, j, k: integer; 
begin 
  if DolA = 0 then begin DolB := 0; exit end;
  k := 1; v[1] := A[1]; w[1] := 1; Predhodnik[1] := 0; 
  for i := 2 to DolA do begin 
    { Poiimo j, dolino najdaljega strogo naraajoega 
      zaporedja s koncem pri A[i]. }
    if v[k] < A[i] then begin k := k + 1; j := k end 
    else begin { j bomo poiskali z bisekcijo. }
      j := k; 
      while j > 1 do
        if v[j - 1] < A[i] then break else j := j - 1;
    end; {if}
    v[j] := A[i]; w[j] := i; 
    if j = 1 then Predhodnik[i] := 0 
    else Predhodnik[i] := w[j - 1]; 
    {WriteLn('A[', i, '] = ', A[i], ', L = ', j, ', k = ', k, ', predhodnik je A[', Predhodnik[i], '] -- ', v[1], ' ', v[2], ' ', v[3]);}
  end; {for}
  { Rekonstruirajmo najdalje strogo naraajoe zaporedje. } 
  DolB := k; i := w[k]; 
  for j := k downto 1 do 
    begin B[j] := A[i]; i := Predhodnik[i] end; 
end; {Najdaljse}

{ Reitev z bisekcijo -- asovna zahtevnost O(n log k). }
procedure Najdaljse2(A: Zaporedje; DolA: integer;
                     var B: Zaporedje; var DolB: integer);
var v, w, Predhodnik: Zaporedje; i, j, h, p, k: integer; 
begin 
  if DolA = 0 then begin DolB := 0; exit end;
  k := 1; v[1] := A[1]; w[1] := 1; Predhodnik[1] := 0; 
  for i := 2 to DolA do begin 
    { Poiimo j, dolino najdaljega strogo naraajoega 
      zaporedja s koncem pri A[i]. }
    if v[k] < A[i] then begin k := k + 1; j := k end 
    else begin { j bomo poiskali z bisekcijo. }
      p := 0; j := k; 
      while j - p > 1 do begin
        { Na tem mestu velja v[p] < A[i] <= v[j].
          Za v[0] si mislimo, da je enak -neskonno. }
        h := (p + j) div 2; 
        { Ker je p >= 0 in j - p >= 2, se ni treba bati, da bi bil h = 0. }
        if v[h] < A[i] then p := h else j := h; 
      end; {while}
      { Zdaj je v[j - 1] < A[i] <= v[j]. }
    end; {if}
    v[j] := A[i]; w[j] := i; 
    if j = 1 then Predhodnik[i] := 0 
    else Predhodnik[i] := w[j - 1]; 
  end; {for}
  { Rekonstruirajmo najdalje strogo naraajoe zaporedje. } 
  DolB := k; i := w[k]; 
  for j := k downto 1 do 
    begin B[j] := i; i := Predhodnik[i] end; 
end; {Najdaljse2}

{ Reitev z malo druganim postopkom rekonstrukcije najdaljega
  strogo naraajoega zaporedja.  L[i] je dolina najdaljega
  s. n. podzaporedja s koncem pri A[i].  asovna zahtevnost je
  e vedno O(n log k). }
procedure Najdaljse3(A: Zaporedje; DolA: integer;
                     var B: Zaporedje; var DolB: integer);
var v, L: Zaporedje; i, j, h, p, k: integer; 
begin 
  if DolA = 0 then begin DolB := 0; exit end;
  k := 1; v[1] := A[1]; L[1] := 1;
  for i := 2 to DolA do begin 
    { Poiimo j, dolino najdaljega strogo naraajoega 
      zaporedja s koncem pri A[i]. }
    if v[k] < A[i] then begin k := k + 1; j := k end 
    else begin { j bomo poiskali z bisekcijo. }
      p := 0; j := k; 
      while j - p > 1 do begin
        { Na tem mestu velja v[p] < A[i] <= v[j].
          Za v[0] si mislimo, da je enak -neskonno. }
        h := (p + j) div 2; 
        { Ker je p >= 0 in j - p >= 2, se ni treba bati, da bi bil h = 0. }
        if v[h] < A[i] then p := h else j := h; 
      end; {while}
      { Zdaj je v[j - 1] < A[i] <= v[j]. }
    end; {if}
    v[j] := A[i]; L[i] := j;
  end; {for}
  { Rekonstruirajmo najdalje strogo naraajoe zaporedje. } 
  DolB := k; j := k; i := DolA;
  while j > 0 do 
    if L[i] = j then begin B[j] := A[i]; j := j - 1 end
    else i := i - 1;
end; {Najdaljse3}

{ Ta reitev ne hrani tabele v[j] z najmanjim doslej prebranim 
  elementom, pri katerem se konuje kakno strogo naraajoe 
  zaporedje doline v.  Namesto tega hrani le tabelo L[i] z
  dolino najdaljega s. n. podzaporedja s koncem pri A[i]. 
  asovna zahtevnost je O(n^2).  Ker je v praksi k << n, je to
  precej slabe od gornjih algoritmov. } 
procedure Najdaljse4(A: Zaporedje; DolA: integer; 
                    var B: Zaporedje; var DolB: integer);  
var L: Zaporedje; i, j, Dol, MaxDol: integer; 
begin
  MaxDol := 0; 
  for i := 1 to DolA do begin 
    Dol := 0; 
    for j := i - 1 downto 1 do if A[i] > A[j] then 
       { Naraajoe zaporedje z zaetkom pri A[i] se lahko 
         nadaljuje pri elementu A[j].  Poglejmo, e bi bilo
         takno zaporedje lahko dalje od najdaljega doslej znanega. }
      if Dol < L[j] then Dol := L[j]; 
    L[i] := Dol + 1; 
    if MaxDol < L[i] then MaxDol := L[i]; 
  end; {for}
  { Rekonstruirajmo najdalje s. n. podzaporedje. }
  i := DolA; Dol := MaxDol; 
  while Dol > 0 do 
    if L[i] = Dol then begin B[Dol] := A[i]; Dol := Dol - 1 end
    else i := i - 1; 
  DolB := MaxDol; 
end; {Najdaljse4} 

{ Izboljava reitve Najdaljse4.  Nad tabelo L zgradimo e manje tabele,
  ki povedo najmanjega izmed dveh, tirih, osmih... zaporednih elementov
  tabele L.  asovna zahtevnost je tako O(n log n), je pa postopek
  malo bolj zapleten. }
procedure Najdaljse5(A: Zaporedje; DolA: integer; 
                     var B: Zaporedje; var DolB: integer); 
var 
  P: Zaporedje;  { P[i] je poloaj elementa A[i] v naraajoem vrstnem redu } 
  PI: Zaporedje; { PI[i] je indeks i-tega najmanjega elementa v tabeli A[i] } 
  
  procedure KeepMax(var A: integer; B: integer); 
    begin if B > A then A := B; end;
    
  { Uredi tabelo PI[l..r] tako, da bo A[PI[i]] <= A[PI[i + 1]]
    za vsak i od l do r-1. }
  procedure QuickSort(l, r: integer);  
  var i, j, t, M: integer; 
  begin
    while l < r do begin 
      M := A[PI[(l + r) div 2]]; i := l; j := r; 
      while i < j do begin 
        while A[PI[i]] < M do i := i + 1; 
        while A[PI[j]] > M do j := j - 1; 
        if i <= j then begin
          t := PI[i]; PI[i] := PI[j]; PI[j] := t;  
          i := i + 1; j := j - 1; 
        end; {if}
      end; {while}
      if j - l < r - i then begin QuickSort(l, j); l := i; end
      else begin QuickSort(i, r); r := j; end;
    end; {while}
  end; {QuickSort}

const MaxCeilLgN = 32;   
var 
  { Za i = 1..DolA je L[i] dolina najdaljega naraajoega zaporedja v A, ki se kona 
    pri elementu A[PI[i]].  V preostanku tabele L so razni maksimumi vrednosti L[i]  
    po bloku 2^k sosednjih i.  Pri vsakem k je DolL[k] = ceil(DolA / 2^k) takih maksimumov,
    zanejo pa se v L[ZacL[k]].  Tako je v celici L[ZacL[k] + j - 1] najveja izmed
    vrednosti L[i] za j 2^k < i <= (j + 1) 2^k. }
  L: array[1..2 * n - 2 + MaxCeilLgN] of integer; 
  DolL, ZacL: array[0..MaxCeilLgN] of integer; 
  JJ: array[1..n] of integer; { JJ[i] elementov v A je strogo manjih od A[i] } 
  CeilLgN, i, j, k, Dol: integer; 
begin
  { Pripravimo ogrodje tabele L. } 
  CeilLgN := 1; while 1 shl CeilLgN < DolA do CeilLgN := CeilLgN + 1; 
  for k := 0 to CeilLgN do DolL[k] := (DolA + (1 shl k) - 1) div (1 shl k); 
  ZacL[0] := 1; for k := 1 to CeilLgN do ZacL[k] := ZacL[k - 1] + DolL[k - 1]; 
  for k := 0 to CeilLgN do for i := 0 to DolL[k] - 1 do L[ZacL[k] + i] := 0;  
  { Uredimo podatke.  Pravzaprav bomo uredili le indekse v PI, ne pa tabele A. } 
  for i := 1 to DolA do PI[i] := i; 
  QuickSort(1, DolA); 
  for i := 1 to DolA do P[PI[i]] := i; { P je le obratno kazalo od PI. } 
  { S pomojo tabele PI lahko za vsak element ugotovimo, koliko je manjih od njega. } 
  JJ[PI[1]] := 0; { od najmanjega ni manji noben element } 
  for i := 2 to DolA do 
    { Od i-tega najmanjega jih je manjih i - 1, razen e ni sluajno  
      (i - 1)-vi najmanji v resnici enak i-temu najmanjemu; tedaj
      je enako mnogo elementov manjih od obeh. }
    if A[PI[i - 1]] = A[PI[i]] then JJ[PI[i]] := JJ[PI[i - 1]] 
    else JJ[PI[i]] := i - 1; 
  { Izraunajmo L[i]. } 
  for i := 1 to DolA do begin 
    { V tabelah L so elementi A[j] za j < i; za ostale j so notri nile. Zanima nas   
      najdalje zaporedje do j po vseh tistih j, za katere je j < i in A[j] < A[i];  
      no, ker so v L za j >= i e nile, je dovolj, e vzamemo maksimum po vseh j, 
      ki imajo A[j] < A[i], ne glede na to, ali so ti j manji ali veji od i. 
      Vrednost JJ[i] nam pove, koliko elementov tabele A je manjih od A[i]. }
    j := JJ[i]; Dol := 0; 
    if j > 0 then 
      { Zanima nas Dol := max(L[1], ..., L[j]).  Zaporedje 1..j si mislimo sestavljeno
        iz blokov velikosti 2^k.  Na primer: j = 23 razdelimo na 1..16, 17..20,
        21..22 in 23..23.  V L imamo e maksimume po vseh teh blokih in Dol bo kar 
        maksimum teh maksimumov.  Blok velikosti 2^k bomo potrebovali,
        e je v j prigan bit k; j shr k pa nam tedaj pove, katera celica v ustreznem   
        delu tabele L predstavlja ta blok.  Pri j = 23 in k = 2 (kar ustreza bloku  
        17..20) na primer dobimo j shr k = 5 in blok 17..20 je res peti po vrsti med  
        bloki velikosti 2^k (pred njim so e tirje: 1..4, 5..8, 9..12 in 13..16). }
      for k := CeilLgN downto 0 do if (j and (1 shl k)) <> 0 then 
        KeepMax(Dol, L[ZacL[k] + (j shr k) - 1]); 
    Dol := Dol + 1; 
    { Popravimo maksimume v tabeli L.  Spremenil se bo L[P[i]] in e vsi
      maksimumi nad njim.  Za P[i] = 23 bi se na primer spremenili maksimumi
      blokov 23..23, 23..24, 21..24, 17..24, 17..32, 1..32, 1..64 itd. }
    j := P[i] - 1; 
    for k := 0 to CeilLgN do KeepMax(L[ZacL[k] + j shr k], Dol); 
  end; {for}
  { Pripravimo elemente, ki tvorijo najdalje naraajoe zaporedje v A. } 
  Dol := L[ZacL[CeilLgN]]; i := DolA; DolB := Dol; 
  while Dol > 0 do 
    if L[P[i]] = Dol then 
      begin B[Dol] := A[i]; Dol := Dol - 1; end
    else i := i - 1; 
end; {Najdaljse5} 

function GetCpuTime: double;
var ct, et, kt, ut: FILETIME; li1, li2: LARGE_INTEGER;
begin
  GetProcessTimes(GetCurrentProcess, ct, et, kt, ut);
  li1.LowPart := ut.dwLowDateTime; li1.HighPart := ut.dwHighDateTime;
  li2.LowPart := ut.dwLowDateTime; li2.HighPart := ut.dwHighDateTime;
  GetCpuTime := 0.0000001 * (li1.QuadPart + li2.QuadPart);
end; {GetCpuTime}

var 
  A, B: Zaporedje; DolA, DolB, Alg, i: integer; 
  StartT, TotalT, DolBSum: double; nIter: integer;
begin {NajdaljseNarascajocePodzaporedje}
  if ParamCount >= 1 then begin
    Val(ParamStr(1), DolA, i); WriteLn('Dolina vhodnega zaporedja: ', DolA); 
  end else begin
    Write('Vnesi dolino vhodnega zaporedja: '); ReadLn(DolA);
  end; {if} 
  if ParamCount >= 2 then Val(ParamStr(2), Alg, i)
  else Alg := 1;
  if (Alg < 0) or (Alg > 5) then Alg := 1;

  { Zaradi natannejega merjenja asa bomo izbrani podprogram
    izvedli na ve nakljunih zaporedjih, dokler ne bo skupni
    as CPU presegel 10 sekund. }
  nIter := 0; TotalT := 0; DolBSum := 0;
  RandSeed := 0; 
  repeat
    for i := 1 to DolA do A[i] := Random(1000000000);
    StartT := GetCpuTime;
    case Alg of
    0: begin end;
    1: Najdaljse(A, DolA, B, DolB);
    2: Najdaljse2(A, DolA, B, DolB);
    3: Najdaljse3(A, DolA, B, DolB);
    4: Najdaljse4(A, DolA, B, DolB);
    5: Najdaljse5(A, DolA, B, DolB);
    end; {case}
    nIter := nIter + 1; 
    TotalT := TotalT + (GetCpuTime - StartT);
    DolBSum := DolBSum + DolB;
    
    if nIter = 1 then begin 
      for i := 1 to DolB - 1 do if B[i] >= B[i + 1] then Write('#');
      Write('Algoritem ', Alg, ' vrne: ', DolB);
    end; {if}
  until TotalT > 10.0;

  WriteLn('   nIter = ', nIter, ', T = ', (TotalT / nIter):0:6, ' s');
  WriteLn('Povprena dolina najdaljega strogo naraajoega podzaporedja: ',
    (DolBSum / nIter):0:2, ' (priakovana: ', (2 * Sqrt(DolA)):0:2, ').');
  
  {for i := 1 to DolB do Write(' ', B[i]);}
end. {NajdaljseNarascajocePodzaporedje}

{ Meritve na procesorju PIII 800:
                n = 1000    10000       100000     1000000
  - as izvajanja v sekundah:
  Algoritem 1:    368 us   10.94 ms     328 ms     10.07    
  Algoritem 2:    373       4.99         68.87      0.790   
  Algoritem 3:    392       4.90         69.07      0.804  
  Algoritem 4:  18142    1822        395869    
  Algoritem 5:   2699      35.63        630.91      9.64
  - Povprena dolina najdaljega strogo naraajoega podzaporedja:
                 58.15      192.21      621.29     1983.85
  - Dobljena kot povpreje po toliko nakljunih tabelah A
                 27185      2044         146         13
  - Priakovana dolina najdaljega strogo naraajoega podzaporedja (2 sqrt n):
                 63.25      200         632.46     2000 }
