program Dynascope(Output); { RTK 1993.3.3 }
const MaxDolzinaSledi = 100; { najveja dolina shranjene sledi izvajanja } 
type 
  BesedaT = 0..65535; 
  UkazT = (LDA, LDAI, STA, STAI, MSUB, MSUBI, JPOS, JPOSI); 
  tpT = (tpAP, tpMNP, tpP); 
  PopravekT = record               { podatki, ki jih spreminjajo ukazi } 
    P: BesedaT;                    { vedno shranimo programski tevec } 
    case TipPopravka: tpT of 
      tpAP: (A: BesedaT);          { shranimo tudi akumulator } 
      tpMNP: (M, Naslov: BesedaT); { shranimo naslov in vsebino pomn. celice } 
      tpP: ( );                    { shranimo le programski tevec } 
  end; {PopravekT}

var 
  M: array[BesedaT] of BesedaT;    { pomnilnik } 
  P: BesedaT;                      { programski tevec } 
  A: BesedaT;                      { akumulator } 
  Sled: array[1..MaxDolzinaSledi] of PopravekT; { sled izvajanja } 
  DolzinaSledi: integer;           { tevilo shranjenih ukazov v tabeli Sled } 

procedure Napaka(Sporocilo: string); 
  begin WriteLn(Sporocilo); Halt; end; {Napaka}

procedure PredUkazom; 
var Korak: PopravekT; 
    Ukaz, Operand: BesedaT; 
begin
  Ukaz := M[P]; Operand := M[P + 1]; 
  Korak.P := P; { vsak ukaz spremeni vsaj P } 
  case Ukaz of 
    Ord(LDA), Ord(LDAI), Ord(MSUB), Ord(MSUBI): { ukazi spremenijo A in P } 
      begin Korak.TipPopravka := tpAP; Korak.A := A; end;
    Ord(STA), Ord(STAI): begin { ukaza spremenita pomnilniko celico in P } 
      Korak.TipPopravka := tpMNP; 
      if Ukaz = Ord(STA) then Korak.Naslov := Operand
      else Korak.Naslov := M[Operand];
      Korak.M := M[Korak.Naslov];
    end; 
    Ord(JPOS), Ord(JPOSI): Korak.TipPopravka := tpP; { ukaza spremenita le P } 
    else Napaka('neznan ukaz'); 
  end; {case} 
  if DolzinaSledi >= MaxDolzinaSledi then Napaka('sled predolga'); 
  DolzinaSledi := DolzinaSledi + 1; { podaljamo sled izvajanja } 
  Sled[DolzinaSledi] := Korak;      { spremembo stanja dodamo v sled } 
end; {PredUkazom} 

procedure UkazNazaj; 
var Korak: PopravekT; 
begin
  if DolzinaSledi < 1 then Napaka('sledi ni'); 
  Korak := Sled[DolzinaSledi];         { vzamemo zadnjo spremembo stanja } 
  DolzinaSledi := DolzinaSledi - 1;    { skrajamo sled izvajanja } 
  P := Korak.P;                        { vedno popravimo P } 
  case Korak.TipPopravka of 
    tpAP: A := Korak.A;                { popravimo A } 
    tpMNP: M[Korak.Naslov] := Korak.M; { popravimo pomnilniko celico } 
    tpP: ;                             { P smo e popravili } 
  end; {case} 
end; {UkazNazaj} 

{-----------------------------------------------------------------------------}
{ Testiranje gornjih podprogramov.                                            }
{-----------------------------------------------------------------------------}

type
  StanjeT = record
    M: array[BesedaT] of BesedaT;    { pomnilnik } 
    P: BesedaT;                      { programski tevec } 
    A: BesedaT;                      { akumulator } 
  end; {StanjeT}

procedure ShraniStanje(var S: StanjeT);
var i: integer;
begin
  for i := Low(BesedaT) to High(BesedaT) do S.M[i] := M[i]; 
  S.P := P; S.A := A;
end; {ShraniStanje}

function EnakoStanje(var S: StanjeT): boolean;
var i: integer;
begin
  EnakoStanje := false;
  for i := Low(BesedaT) to High(BesedaT) do 
    if S.M[i] <> M[i] then exit;
  if S.P <> P then exit;
  if S.A <> A then exit;
  EnakoStanje := true;
end; {EnakoStanje}

procedure IzvediUkaz;
var Ukaz, Operand: BesedaT;
begin
  Ukaz := M[P]; Operand := M[P + 1]; P := P + 2;
  case Ukaz of 
    Ord(LDA): A := Operand;
    Ord(LDAI): A := M[Operand];
    Ord(MSUB): A := Operand - A;
    Ord(MSUBI): A := M[Operand] - A;
    Ord(STA): M[Operand] := A;
    Ord(STAI): M[M[Operand]] := A;
    Ord(JPOS): if A >= 0 then P := Operand;
    Ord(JPOSI): if A >= 0 then P := M[Operand];
    else Napaka('neznan ukaz'); 
  end; {case} 
end; {IzvediUkaz}

var i: integer; Stanja: array[1..MaxDolzinaSledi] of StanjeT;
begin {Dynascope}
  { Vpiimo v pomnilnik nakljune vrednosti. }
  for i := Low(BesedaT) to High(BesedaT) do M[i] := Random(8);
  A := Random(8); P := 0;
  { Izvedimo nekaj ukazov. }
  for i := 1 to MaxDolzinaSledi do begin
    ShraniStanje(Stanja[i]);
    PredUkazom;
    IzvediUkaz;
  end; {for i}
  { Poskusimo zavrteti as nazaj. }
  for i := MaxDolzinaSledi downto 1 do begin
    UkazNazaj;
    if not EnakoStanje(Stanja[i]) then 
      Napaka('vrnitev v staro stanje ni uspela');
  end; {for}
end. {Dynascope}
