program Krog; { RTK 1993.3.4 }
{ Ta program simulira komunikacijo med procesorji
  in na zaslonu prikazuje, kaj se dogaja.
  # = prizgane (pobarvane) toke; 
  o = procesorji, ki so dobili sporoilo, niso pa se prigali;
  . = ostali procesorji. }
uses Crt;

{ Da bomo laje simulirali dogajanje na vseh procesorjih
  hkrati, smo program, ki naj bi ga izvajal vsak procesor,
  preoblikovali v tale podprogram.  Klical naj bi se ob
  vsakem prejemu sporoila; tako ali tako pa naj noben
  procesor ne bi dobil po ve kot enega sporoila. }
type 
  SporociloT = record 
    r: real; 
    x, y: integer; 
  end;
  
  procedure Poslji(Posiljatelj: integer; Sporocilo: SporociloT; 
                   vSmer: integer); forward; 
  procedure Prizgi(StRac: integer); forward; 

procedure ObSprejemu(StRac: integer; Sprejeto: SporociloT; Smer: integer);

  { Ta podprogram bo tudi prigal nao toko, e res lei v krogu. } 
  function VKrogu: boolean; 
  begin
    if Sprejeto.x * Sprejeto.x + Sprejeto.y * Sprejeto.y > Sprejeto.r 
      then VKrogu := false
      else begin Prizgi(StRac); VKrogu := true; end; 
  end; {VKrogu} 

var Stevec: integer;
begin
  case Smer of 
    0: begin{ sredie kroga } 
      Sprejeto.x := 0; Sprejeto.y := 0; 
      Sprejeto.r := Sprejeto.r * Sprejeto.r; 
      Prizgi(StRac); 
      for Stevec := 1 to 4 do Poslji(StRac, Sprejeto, Stevec); 
    end;
    2, 4: begin{ vzhodno ali zahodno od sredia } 
      Sprejeto.x := Sprejeto.x + Smer - 3; 
      if VKrogu then { obvestimo tri sosede } 
        for Stevec := 1 to 4 do if Stevec <> Smer then 
          Poslji(StRac, Sprejeto, Stevec); 
    end;
    1, 3: begin{ severno ali juno } 
      Sprejeto.y := Sprejeto.y + Smer - 2; 
      if VKrogu then Poslji(StRac, Sprejeto, 4 - Smer); 
    end;
  end; {case} 
end; {Krog} 

const 
  Sever = 1; Vzhod = 2; Jug = 3; Zahod = 4;
  CX = 80; CY = 50; { velikost polja procesorjev }
type
  RacunalnikT = record
    Cas, KdajPride, OdKodPride: integer;
    S: SporociloT;
  end; {if}
var Rac: array[0..CY - 1, 0..CX - 1] of RacunalnikT;

procedure Poslji(Posiljatelj: integer; Sporocilo: SporociloT; 
                 vSmer: integer); 
const 
  DX: array[1..4] of integer = (0, 1, 0, -1);
  DY: array[1..4] of integer = (-1, 0, 1, 0);
  Nasprotna: array[1..4] of integer = (Jug, Zahod, Sever, Vzhod);
var x, y, xx, yy: integer;
begin
  x := Posiljatelj mod CX; y := Posiljatelj div CX;
  xx := x + DX[vSmer]; yy := y + DY[vSmer]; 
  if (xx < 0) or (yy < 0) or (xx >= CX) or (yy >= CY) then exit;
  { Recimo, da se poiljatelj s poiljanjem zakasni
    za eno asovno enoto.  Tudi prejemnik bo sporoilo
    videl ele ez eno asovno enoto. }
  with Rac[y, x] do Cas := Cas + 1;
  with Rac[yy, xx] do begin
    Assert(KdajPride < 0);
    S := Sporocilo;
    KdajPride := Rac[y, x].Cas;
    OdKodPride := Nasprotna[vSmer];
  end; {with}
end; {Poslji}

procedure Prizgi(StRac: integer); 
begin
  GotoXY(1 + (StRac mod CX), 1 + (StRac div CX));
  Write('#');
end; {Prizgi}

var x, y, x0, y0, T: integer;
begin
  { Inicializacija. }
  ClrScr; GotoXY(1, 1);
  for y := 0 to CY - 1 do for x := 0 to CX - 1 do begin
    with Rac[y, x] do KdajPride := -1;
    Write('.'); if x = CX - 1 then WriteLn;
  end; {for x, y}
  { Poljimo zaetno sporoilo procesorju v srediu kroga. }
  x0 := 57; y0 := 30;
  with Rac[y0, x0] do 
    begin KdajPride := 0; OdKodPride := 0; S.r := 15.5; end;
  { Simulirajmo dogajanje. }
  T := 0;
  while not KeyPressed do begin
    for y := 0 to CY - 1 do for x := 0 to CX - 1 do 
      with Rac[y, x] do if KdajPride = T then begin
        GotoXY(1 + x, 1 + y); Write('o');
        Cas := T;
        ObSprejemu(y * CX + x, S, OdKodPride);
      end; {with}
    T := T + 1; Delay(500); 
  end; {while}
end. {RacunalnikT}
