program OdkrivanjePokvarjenih; { RTK 1995.1.4 }
const MaxN = 50; 
var 
  Pokvarjen: array[1..MaxN] of boolean; 
  Enaka: array[1..MaxN, 1..MaxN] of boolean;

procedure Preverjaj(N: integer); 
var i, j, Delujocih: integer; 
begin
  j := 0; 
  repeat 
    j := j + 1; 
    { Na tem mestu e vemo, da so vsi procesorji od 1 do j - 1 pokvarjeni 
      (sicer bi se tale zanka e konala).  O procesorju j zaenkrat predpostavimo,
      da je tudi pokvarjen.  Spremenljivka Delujocih teje v resnici to, koliko 
      procesorjev (ne vtevi j-ja samega) meni, da j deluje pravilno. }
    Pokvarjen[j] := true; Delujocih := 0; 
    { Poglejmo zdaj, kaj pravijo o j-ju ostali procesorji.  
      Tistih od 1 do j - 1 nima smisla spraevati, ker vemo, da so pokvarjeni. }
    for i := j + 1 to N do 
      if Enaka[j, i] then begin 
        { i meni, da j deluje pravilno.  e se bo na koncu izkazalo, da j deluje pravilno, 
          sledi iz tega, da tudi i deluje pravilno, zato lahko e zdaj postavimo 
          Pokvarjen[i] na false.  e se bo izkazalo, da je j vendarle pokvarjen, 
          bomo Pokvarjen[i] kasneje lahko e popravili. }
        Pokvarjen[i] := false; Delujocih := Delujocih + 1 
      end else begin 
        { i meni, da je j pokvarjen.  e bi se izkazalo, da deluje j pravilno, }
        Pokvarjen[i] := true;                      { sledi, da je i pokvarjen. }
      end; {if}
    { Vemo, da je pokvarjenih manj kot pol procesorjev.  Loimo naslednje monosti: 
      N = 2K + 1 (lih), torej vsaj K + 1 dobrih, najve K slabih. 
          j pokvarjen --> vsaj K + 1 jih bo reklo, da je zani; 
                          torej jih bo najve K - 1 reklo, da je dober. 
          j dober     --> vsaj K jih bo reklo, da je dober. 
      N = 2K (sod), torej vsaj K + 1 dobrih, najve K - 1 slabih. 
          j pokvarjen --> vsaj K + 1 jih bo reklo, da je zani; 
                          torej jih bo najve K - 2 bo reklo, da je dober. 
          j dober     --> vsaj K jih bo reklo, da je dober. 
      Torej je j dober natanko tedaj, ko je vsaj K = N div 2 drugih
      procesorjev reklo, da je dober. }
  until Delujocih >= N div 2;
  { To, da smo prili do sem, pomeni, da smo ugotovili, da deluje j pravilno. } 
  Pokvarjen[j] := false
  { Zdaj za vse procesorje vemo, ali delujejo pravilno ali ne. } 
end; {Preverjaj} 

var ResPokvarjen: array[1..MaxN] of boolean;

{ Pripravi tabeli Enaka in ResPokvarjen. 
  Tabela Enaka pove, kakne vrednosti bi v nai simulaciji
  vraala funkcija Enaka, ki jo omenja besedilo naloge. }
procedure PripraviPodatke(N, StPokvarjenih: integer);
var i, j: integer; Indeksi: array[1..MaxN] of integer;
begin
  Randomize;
  for i := 1 to N do begin
    j := Random(i) + 1;
    Indeksi[i] := Indeksi[j]; Indeksi[j] := i;
  end; {for i}
  for i := 1 to N do ResPokvarjen[i] := false;
  for i := 1 to StPokvarjenih do
    ResPokvarjen[Indeksi[i]] := true;
  for i := 1 to N do begin
    Enaka[i, i] := true;
    for j := i + 1 to N do begin
      if ResPokvarjen[i] and ResPokvarjen[j] then
        Enaka[i, j] := Random(2) = 0
      else if ResPokvarjen[i] or ResPokvarjen[j] then
        Enaka[i, j] := false
      else
        Enaka[i, j] := true;
      Enaka[j, i] := Enaka[i, j];
    end; {for j}
  end; {for i}
end; {PripraviPodatke}

var N, StPokvarjenih, i: integer;
begin {OdkrivanjePokvarjenih}
  if ParamCount >= 1 then begin
    Val(ParamStr(1), N, i); WriteLn('tevilo vseh procesorjev = ', N);
  end else begin
    Write('Vnesi tevilo procesorjev: '); ReadLn(N);
  end; {if}
  if ParamCount >= 2 then begin
    Val(ParamStr(2), StPokvarjenih, i); WriteLn('tevilo pokvarjenih procesorjev = ', StPokvarjenih);
  end else begin
    Write('Vnesi tevilo pokvarjenih procesorjev: '); ReadLn(StPokvarjenih);
  end; {if}
  
  { Izmislimo si, kateri procesorji so pokvarjeni 
    in kaj bi vraala namiljena funkcija Enaka. }
  PripraviPodatke(N, StPokvarjenih);

  { Poenimo podprogram za odkrivanje pokvarjenih procesorjev. }
  Preverjaj(N);

  { Primerjajmo rezultat s pravimi vrednostmi. }
  Write('Pokvarjeni procesorji:  '); 
  for i := 1 to N do if Pokvarjen[i] then Write(' ', i);
  WriteLn; 
  Write('Pravilni rezultat pa je:');
  for i := 1 to N do if ResPokvarjen[i] then Write(' ', i);
  WriteLn; 
end. {OdkrivanjePokvarjenih}