program Protokol2; { RTK 1996.3.1 }
{ Ta program simulira dogajanje v omreju.
  Pogosto je ve monosti, kaj se bo zgodilo v
  naslednjem hipu; na primer, lahko raunalnik 1
  zahteva poiljanje, lahko pa to stori raunalnik 2.
  Program Protokol1 si je vedno nakljuno izbral
  eno od monih nadaljevanj.  Tale program pa vedno
  sistematino zgenerira vsa mona nadaljevanja 
  in tako sasoma preie vsa mona stanja sistema 
  kot celote (spodaj se jim ree "opisi", da ne pride 
  do zmede z notranjim stanjem posameznega raunalnika).
  Na koncu program izpie, koliko je vseh monih
  opisov; e so pri kaknem teave, izpie cel potek
  dogajanja, ki pripelje do njega. }

const MaxVrsta = 3;
type
  SporociloT = (PosiljalBi, KarDaj, RajeNe, Podatki);
  StanjeT = (Osnovno, RadBiPosiljal, Posiljanje, ZahtevanSprejem, Sprejemanje);
  RacunalnikT = record
    Stanje: StanjeT;
    SprejemVkljucen: boolean;
    Vrsta: array[1..MaxVrsta] of SporociloT;
    Dolzina: integer;
  end; {RacunalnikT}
  OpisT = record
    R: array[1..2] of RacunalnikT;
    Prej: integer;
    S: string;
    Tezava: boolean;
    { TezavaZePrej := Opisi[Prej].Tezava or Opisi[Prej].TezavaZePrej }
    TezavaZePrej: boolean;
  end; {OpisT}
const 
  ImeSporocila: array[SporociloT] of string = ('PosiljalBi', 'KarDaj', 'RajeNe', 'Podatki');
  ImeStanja: array[StanjeT] of string = ('Osnovno', 'RadBiPosiljal', 'Posiljanje', 'ZahtevanSprejem', 'Sprejemanje');
  ImeBooleana: array[Boolean] of string = ('false', 'true');

const MaxOpisov = 1000;
var
  Opisi: array[1..MaxOpisov] of OpisT;
  StOpisov: integer;

function EnakaOpisa(O1, O2: OpisT): boolean;
var StRac, i: integer;
begin
  EnakaOpisa := false;
  for StRac := 1 to 2 do begin
    if O1.R[StRac].Stanje <> O2.R[StRac].Stanje then exit;
    if O1.R[StRac].SprejemVkljucen <> O2.R[StRac].SprejemVkljucen then exit;
    if O1.R[StRac].Dolzina <> O2.R[StRac].Dolzina then exit;
    for i := 1 to O1.R[StRac].Dolzina do
      if O1.R[StRac].Vrsta[i] <> O2.R[StRac].Vrsta[i] then exit;
  end; {for}
  if O1.Tezava <> O2.Tezava then exit;
  EnakaOpisa := true;
end; {EnakaOpisa}

{ Doda opis v tabelo Opisi, e tam e ni takega. }
procedure DodajOpis(Novi: OpisT; Prej: integer; S: string; Tezava: boolean);
var i: integer;
begin
  Novi.Prej := Prej; Novi.S := S; Novi.Tezava := Tezava;
  Novi.TezavaZePrej := Opisi[Prej].Tezava or Opisi[Prej].TezavaZePrej;
  for i := 1 to StOpisov do 
    if EnakaOpisa(Novi, Opisi[i]) then exit;
  Assert(StOpisov < MaxOpisov);
  Inc(StOpisov); Opisi[StOpisov] := Novi;
end; {DodajOpis}

{ Zgenerira vse mone opise, v katere lahko celoten sistem
  pride v enem koraku iz opisa Opisi[StOpisa].  Te mone
  opise doda v tabelo opisov s pomojo podprograma DodajOpis. }
procedure PripraviMoznaNadaljevanja(StOpisa: integer);
var 
  StRac: integer;     { raunalnik, na katerem trenutno simuliramo dogajanje }
  Stari, Novi: OpisT; { Stari = Opisi[StOpisa], Novi = novi opis, do katerega pridemo na razne naine }
  StanjeS,            { niz oblike "[StRac.Stanje]  " }
  PosljiS: string;    { pripravi ga Poslji, kasneje ga uporabimo pri dodajanju novega opisa }
  PosljiOk: boolean;  { to postavi Poslji na false, e je prejemnikova vrsta prepolna }

  { Sprejmi in Poslji oba delata s spremenljivko Novi. }
  
  function Sprejmi: SporociloT;
  var i: integer;
  begin
    with Novi.R[StRac] do begin
      Assert(Dolzina > 0);
      Sprejmi := Vrsta[1];
      for i := 2 to Dolzina do Vrsta[i - 1] := Vrsta[i];
      Dec(Dolzina);
    end; {with}
  end; {Sprejmi}

  procedure Poslji(S: SporociloT);
  begin
    with Novi.R[3 - StRac] do 
      if Dolzina >= MaxVrsta then begin 
        PosljiOk := false; PosljiS := StanjeS + 'Poslal sporoilo ' + ImeSporocila[S] + ', vendar ima drugi raunalnik prepolno vrsto.'; 
      end else begin
        PosljiOk := true; PosljiS := StanjeS + 'Poslal sporoilo ' + ImeSporocila[S] + '.';
        Inc(Dolzina); Vrsta[Dolzina] := S;
      end; {if}
  end; {Poslji}

var S: SporociloT; StRacS, SprejemS: string;
begin {PripraviMoznaNadaljevanja}
  Stari := Opisi[StOpisa];
  for StRac := 1 to 2 do begin
    Str(StRac, StRacS);
    StanjeS := '[' + StRacS + '.' + ImeStanja[Stari.R[StRac].Stanje] + ']  ';
    case Stari.R[StRac].Stanje of

      Osnovno: begin
        
        { Ena monost je, da prevzamemo sporoilo.  
          e je to PosiljalBi, se prestavimo v stanje ZahtevanSprejem;
          e je to kakno drugo sporoilo, pa ga ignoriramo in javimo to kot teavo. }
        if Stari.R[StRac].Dolzina > 0 then begin
          Novi := Stari; S := Sprejmi;
          if S <> PosiljalBi then 
            DodajOpis(Novi, StOpisa, StanjeS + 'IGNORIRAM sporoilo ' + ImeSporocila[S] + '.', true)
          else begin
            Novi.R[StRac].Stanje := ZahtevanSprejem;
            DodajOpis(Novi, StOpisa, StanjeS + 'Prejel sporoilo ' + ImeSporocila[S] + '.', false);
          end; {if}
        end; {if}
        
        { Druga monost je, da spremenimo SprejemVkljucen. }
        Novi := Stari; Novi.R[StRac].SprejemVkljucen := not Novi.R[StRac].SprejemVkljucen;
        DodajOpis(Novi, StOpisa, StanjeS + 'Postavil SprejemVkljucen na ' + ImeBooleana[Novi.R[StRac].SprejemVkljucen] + '.', false);
        
        { Tretja monost je, da si zaelimo sami kaj poslati. }
        Novi := Stari; Novi.R[StRac].Stanje := RadBiPosiljal; 
        Poslji(PosiljalBi); 
        DodajOpis(Novi, StOpisa, PosljiS, not PosljiOk);

    end; {Osnovno}
    
    RadBiPosiljal: begin
      if Stari.R[StRac].Dolzina <= 0 then continue;
      Novi := Stari; S := Sprejmi; 
      Novi.R[StRac].Stanje := Osnovno; 
      case S of
      RajeNe: DodajOpis(Novi, StOpisa, StanjeS + 'Prejel sporoilo ' + ImeSporocila[S] + '.', false); 
      KarDaj: begin 
        Novi.R[StRac].Stanje := Posiljanje; 
        DodajOpis(Novi, StOpisa, StanjeS + 'Prejel sporoilo ' + ImeSporocila[S] + '.', false); end;
      else DodajOpis(Novi, StOpisa, StanjeS + 'IGNORIRAM sporoilo ' + ImeSporocila[S] + ', uporabnikovega sporoila NE BOM POSLAL.', true); 
      end; {case}
    end; {RadBiPosiljal}
    
    Posiljanje: begin
      Novi := Stari;
      Poslji(Podatki); 
      Novi.R[StRac].Stanje := Osnovno; 
      DodajOpis(Novi, StOpisa, PosljiS, not PosljiOK);
    end; {Posiljanje}
    
    ZahtevanSprejem: begin
      Novi := Stari;
      if Novi.R[StRac].SprejemVkljucen then begin 
        Poslji(RajeNe); Novi.R[StRac].Stanje := Osnovno;
      end else begin 
        Poslji(KarDaj); Novi.R[StRac].Stanje := Sprejemanje;
      end; {if}
      DodajOpis(Novi, StOpisa, PosljiS, not PosljiOK);
    end; {ZahtevanSprejem}
    
    Sprejemanje: begin
      if Stari.R[StRac].Dolzina <= 0 then continue;
      Novi := Stari; S := Sprejmi; 
      SprejemS := StanjeS + ' Prejel sporoilo ' + ImeSporocila[S];
      if S <> Podatki then SprejemS := SprejemS + ' (MORALO BI BITI Podatki!)';
      SprejemS := SprejemS + '.'; Novi.R[StRac].Stanje := Osnovno;
      DodajOpis(Novi, StOpisa, SprejemS, S <> Podatki);
    end; {Sprejemanje}
    
    end; {case Stanje}
  end; {with, for StRac}
end; {PripraviMoznaNadaljevanja}

function OpisVNiz(var O: OpisT): string;
var StRac, i: integer; S, T: string;
begin
  S := '';
  for StRac := 1 to 2 do with O.R[StRac] do begin
    Str(StRac, T); S := S + '[' + T + '.' + ImeStanja[Stanje] + '/';
    if SprejemVkljucen then S := S + 'vkljuen' 
    else S := S + 'izkljuen';
    for i := 1 to Dolzina do S := S + ' ' + ImeSporocila[Vrsta[i]];
    S := S + '] ';
  end; {for}
  if O.Tezava then S := S + ' (TEAVA!)';
  OpisVNiz := S;
end; {OpisVNiz}

{ Izpie zaporedje opisov, ki pripeljejo od 
  zaetnega opisa (opis 1) do opisa StOpisa.
  Med vsakima zaporednima dvema opisoma napie e,
  s kaknim dogodkom se pride od enega do drugega. }
procedure PotDoOpisa(StOpisa: integer);
begin
  if StOpisa > 1 then begin
    { Kako se pride do prejnjega opisa v zaporedju
      dogajanja, ki pripelje do trenutnega opisa? }
    PotDoOpisa(Opisi[StOpisa].Prej);
    { Kako se iz prejnjega opisa pride do trenutnega? }
    WriteLn(Opisi[StOpisa].S);
  end; {if}
  { Izpiimo e trenutni opis. }
  WriteLn('(#', StOpisa, ') ', OpisVNiz(Opisi[StOpisa]))
end; {PotDoOpisa}

const Locnica = '-------------------------------------------------';

procedure IzpisiTezave;
var StOpisa: integer;
begin
  for StOpisa := 1 to StOpisov do with Opisi[StOpisa] do
    if Tezava and (not TezavaZePrej) then begin
      WriteLn(Locnica);
      PotDoOpisa(StOpisa);
    end; {if}
end; {IzpisiTezave}

{ Izpie vse doseene opise.  Pri vsakem napie e,
  iz katerega predhodnega opisa je nastal in s kaknim
  dogodkom. }
procedure IzpisiOpise;
var StOpisa: integer;
begin
  for StOpisa := 1 to StOpisov do with Opisi[StOpisa] do begin
    WriteLn('(#', StOpisa, ') ', OpisVNiz(Opisi[StOpisa]));
    if Prej > 0 then WriteLn('     IZ #', Prej, ' ', OpisVNiz(Opisi[Prej]));
    WriteLn('     PO ', S);
  end; {for}
end; {IzpisiOpise}

var StOpisa, StRac: integer;
begin {Protokol2}

  WriteLn('Testiranje protokola');
  
  { Inicializacija zaetnega opisa. }
  with Opisi[1] do begin
    for StRac := 1 to 2 do with R[StRac] do begin 
      Stanje := Osnovno; SprejemVkljucen := true; Dolzina := 0; 
    end; {with, for}
    Prej := 0; Tezava := false; TezavaZePrej := false; S := 'Zaetno stanje sistema.';
  end; {with}
  StOpisov := 1;

  { Simulirajmo vse mone poteke dogajanja v sistemu. }  
  WriteLn('Generiram vse dosegljive opise...');
  StOpisa := 1;
  while StOpisa <= StOpisov do begin 
    { Za opise 1..StOpisa-1 smo e pripravili vsa mona
      nadaljevanja (vse opise, do katerih se da priti iz
      njih v enem samem koraku).  Naredimo zdaj to e
      za opis StOpisa. }
    PripraviMoznaNadaljevanja(StOpisa);
    Inc(StOpisa);
  end; {while}
  WriteLn('tevilo dosegljivih opisov: ', StOpisov);

  WriteLn(Locnica);
  IzpisiOpise;  
  WriteLn(Locnica);
  WriteLn('e je kaj teav, sledijo spodaj.');
  IzpisiTezave;
end. {Protokol2}
