program Rozine3; { RTK 1997.Z.3 }
{ Do soseine pride tako, da ravnino pometa v smeri X
  s pasom irine A; toke, ki so trenutno v pasu, hrani
  v dvojikem iskalnem drevesu, urejene po Y-koordinatah.
  Tako je iskanje soseine res zelo uinkovito. }
  
const MaxN = 10000; 
type 
  TabelaI = array[1..MaxN] of integer; 
  TabelaR = array[1..MaxN] of real; 
  PTabelaI = ^TabelaI; PTabelaR = ^TabelaR; 
var   
  PtX, PtY: ^TabelaR;  { koordinate tok } 
  N: integer;          { tevilo tok } 
  A, B: real;          { velikost poizvedovalnega okna } 
  { binarno iskalno drevo, ki vsebuje toke trenutnega pasu } 
  Left, Right, Parent: ^TabelaI; Root: integer; { urejene so po Y-koordinatah } 
  { toke, urejene po eni od koordinat }  
  XOrder, YOrder: ^TabelaI; { v XOrder so vse toke, v YOrder pa neka okolica } 
  
  { Uredi elemente tabele Order^[L..R].  Elementi naj bodo indeksi v 
    tabelo Coord, primerja pa se jih po ustrezni vrednosti v Coord. }
  procedure QuickSort(Coord: PTabelaR; Order: PTabelaI; L, R: integer); 
  var i, j, T: integer; M: real; 
  begin
    while L < R do begin 
      M := Coord^[Order^[(L + R) div 2]]; i := L; j := R; 
      while i < j do begin 
        while Coord^[Order^[i]] < M do Inc(i); 
        while Coord^[Order^[j]] > M do Dec(j); 
        if i <= j then begin T := Order^[i]; Order^[i] := Order^[j]; 
          Order^[j] := T; Inc(i); Dec(j); end;
      end; {while}
      if j - L < R - i then begin QuickSort(Coord, Order, L, j); L := i; end
      else begin QuickSort(Coord, Order, i, R); R := j; end;
    end; {while}
  end; {QuickSort} 
  
  { V Dest doda (in povea Count) vsa vozlia (v poddrevesu, ki se  
    zane pri Node), ki imajo Y-koordinato vsaj YMin in najve YMax. }
  procedure TreeQuery(Dest: PTabelaI; var Count: integer; 
                      YMin, YMax: real; Node: integer); 
  var YNode: real; 
  begin
    YNode := PtY^[Node];  
    if (Left^[Node] > 0) and (YMin <= YNode) then 
      TreeQuery(Dest, Count, YMin, YMax, Left^[Node]); 
    if (YMin <= YNode) and (YNode <= YMax) then 
      begin Inc(Count); Dest^[Count] := Node; end;  
    if (Right^[Node] > 0) and (YNode <= YMax) then 
      TreeQuery(Dest, Count, YMin, YMax, Right^[Node]); 
  end; {TreeQuery} 
  
  procedure TreeAdd(Node: integer); { Doda vozlie Node v drevo. } 
  var P: integer; YParent, YNode: real; 
  begin
    Left^[Node] := 0; Right^[Node] := 0;  
    if Root = 0 then begin Root := Node; exit; end;
    P := Root; YNode := PtY^[Node]; 
    repeat 
      YParent := PtY^[P];  
      if YNode < YParent then begin
        if Left^[P] = 0 then begin Left^[P] := Node; break; end;
        P := Left^[P]; 
      end else begin 
        if Right^[P] = 0 then begin Right^[P] := Node; break; end;
        P := Right^[P]; 
      end; {if}
    until false; 
    Parent^[Node] := P; 
  end; {TreeAdd} 

  procedure TreeDel(Node: integer); { Zbrie vozlie Node iz drevesa. } 
  var LC, RC, Next, P: integer; 
  begin
    LC := Left^[Node]; RC := Right^[Node]; P := Parent^[Node]; 
    if (LC = 0) or (RC = 0) then begin
      if LC = 0 then Next := RC else Next := LC; 
    end else begin 
      Next := RC; 
      while Left^[Next] > 0 do Next := Left^[Next]; 
      TreeDel(Next);  
      RC := Right^[Node]; 
      Parent^[LC] := Next; if RC > 0 then Parent^[RC] := Next; 
      Left^[Next] := LC; Right^[Next] := RC; 
    end; {if}
    if Next > 0 then Parent^[Next] := P;  
    if P = 0 then Root := Next 
    else if Left^[P] = Node then Left^[P] := Next 
    else { Right^[P] = Node } Right^[P] := Next; 
  end; {TreeDel} 

var
  i, Best: integer; F: text;  
  X0, Y0: real;          { koordinati trenutne toke; X0 doloa levi rob pasu } 
  Y1: real;              { trenutna koordinata spodnjega roba pravokotnika } 
  XFrom, XTo: integer;   { v pasu so toke XOrder^[XFrom..XTo - 1] } 
  YCount: integer;       { koliko tok pasu ima Y-koordinato blizu Y0 } 
  YFrom, YTo: integer;   { trenutni pravokotnik X0 <= X <= X0 + A, Y1 <= Y <= Y0 + B 
                           pokriva toke YOrder^[YFrom..YTo - 1] }
begin {Rozine3} 

  { Preberimo vhodne podatke. } 
  New(Left); New(Right); New(Parent); 
  New(XOrder); New(YOrder); New(PtX); New(PtY); 
  if ParamCount >= 1 then Assign(F, ParamStr(1))
  else Assign(F, 'input.txt'); 
  Reset(F); ReadLn(F, A, B); ReadLn(F, N); 
  for i := 1 to N do ReadLn(F, PtX^[i], PtY^[i]);  
  Close(F); 
  
  { Uredimo toke po naraajoem X. } 
  for i := 1 to N do XOrder^[i] := i; 
  QuickSort(PtX, XOrder, 1, N); 
  
  { Drevo je na zaetku prazno.  Pometajmo ravnino.} 
  { Zaetni poloaj levega roba pasu je tak, da gre skozi najbolj levo toko. } 
  Best := 1; XFrom := 0; XTo := 1; Root := 0; 
  while XFrom <= N - Best do begin 

    { Premaknimo levi rob pasu do naslednje toke (proti desni).
      Prejnja zato izpade iz pasu, lahko pa vanj pride nekaj novih. }
    if XFrom > 0 then TreeDel(XOrder^[XFrom]); 
    Inc(XFrom); X0 := PtX^[XOrder^[XFrom]]; 
    while XTo <= N do begin 
      if PtX^[XOrder^[XTo]] > X0 + A then break; 
      TreeAdd(XOrder^[XTo]); Inc(XTo); 
    end; {while}
    if XTo - XFrom <= Best then continue; { Brezupno. } 
    
    { Zanimali nas bodo pravokotniki, ki se z levo stranico dotikajo 
      toke XOrder^[XFrom].  Pripravimo si seznam tok,
      ki so v pasu in imajo Y0 - B <= Y <= Y0 + B. }
    Y0 := PtY^[XOrder^[XFrom]]; YCount := 0; 
    TreeQuery(YOrder, YCount, Y0 - B, Y0 + B, Root); 
    if YCount <= Best then continue; { Brezupno. } 
    
     { Preletimo ta seznam z intervalom irine B. } 
    YFrom := 0; YTo := 1;  
    while (YFrom <= YCount - Best) and (YTo <= YCount) do begin 
      { Interval bo pokrival koordinate od Y1 do Y1 + B. } 
      Inc(YFrom); Y1 := PtY^[YOrder^[YFrom]]; 
      while YTo <= YCount do 
        if PtY^[YOrder^[YTo]] > Y1 + B then break else Inc(YTo); 
      { Na intervalu so toke YOrder^[YFrom..YTo - 1]. } 
      if YTo - YFrom > Best then Best := YTo - YFrom; 
    end; {prelet seznama YOrder z intervalom viine B} 

  end; {prelet ravnine s pasom irine A} 

  { Izpiimo rezultat in poistimo pomnilnik. } 
  if ParamCount >= 2 then Assign(F, ParamStr(2))
  else Assign(F, 'output.txt'); 
  Rewrite(F); WriteLn(F, Best); Close(F); 
  Dispose(Left); Dispose(Right); Dispose(Parent); 
  Dispose(XOrder); Dispose(YOrder); Dispose(PtX); Dispose(PtY); 
end. {Rozine3} 
