program UnijaPravokotnikov; { RTK 1998.2.3 }
{ Program za raunanje povrine unije ve pravokotnikov,
  katerih stranice so vzporedne s koordinatnima osema.
  Vhodni in izhodni podatki so v obliki, kot jo
  doloa naloga A ("Atlantis") z ACMovega "Mid-Central
  European Regional Contest" (Freiburg, 19. nov. 2000).
  http://acm.baylor.edu/Past/icpc2001/Regionals/MCEurope00/Problems.pdf
  http://www.acm.inf.ethz.ch/ProblemSetArchive/B_EU_MCRC/2000/ (tudi testni primeri)
  Glej tudi acm.uva.es/archive, #2184.
  http://acm.uva.es/archive/data/2000eu25.php }

const MaxN = 100; MaxCeilLg2N = 8;
type 
  float = real;
  KoordinateT = array[1..2 * MaxN] of float;
  KazaloT = array[1..2 * MaxN] of integer;
  
var N: integer; 

  { V tabeli Kazalo pripravi indekse od 1 do 2 * N,
    vendar premeane tako, da je k[Kazalo[i]] <= k[Kazalo[i + 1]]
    za vsak i od 1 do 2 * N - 1.  e je ve koordinat
    enakih, pridejo prej v vrsti red tiste z manjim indeksom v k.
    To bo zagotovilo, da npr. pri izrojenem pravokotniku
    s irino ali viino 0 vendarle zagotovo obiemo levo 
    (ali spodnjo) stranico prej kot desno (ali zgornjo). }
  procedure UrediKoordinate(var k: KoordinateT; var Kazalo: KazaloT);
  
    procedure QuickSort(L, R: integer);
    var m: float; t, i, j, mi: integer;
    begin
      while L < R do begin
        mi := Kazalo[(L + R) div 2]; m := k[mi];
        i := L; j := R;
        while i < j do begin
          while (k[Kazalo[i]] < m) or ((k[Kazalo[i]] = m) and (Kazalo[i] < mi)) do Inc(i);
          while (k[Kazalo[j]] > m) or ((k[Kazalo[j]] = m) and (Kazalo[j] > mi)) do Dec(j);
          if i <= j then begin
            t := Kazalo[i]; Kazalo[i] := Kazalo[j]; Kazalo[j] := t;
            Inc(i); Dec(j);
          end; {if}
        end; {while i < j}
        if j - L < R - i then begin
          QuickSort(L, j); L := i;
        end else begin
          QuickSort(i, R); R := j;
        end; {if}
      end; {while}
    end; {QuickSort}
  
  var i: integer;
  begin {UrediKoordinate}
    for i := 1 to 2 * N do Kazalo[i] := i;
    QuickSort(1, 2 * N);
  end; {UrediKoordinate}
  
var 
  { Drevo segmentov.  i-to vozlie nivoja j je Drevo[Zacetek[j] + i - 1].
    Nivo j ima Sirina[j] vozli.  Vseh nivojev je Globina.
    Nivo 1 so listi, nivo Globina je koren. }
  Drevo: array[1..4 * MaxN + MaxCeilLg2N] of integer;
  { Pokrito[j] je dolina unije vseh pokritih intervalov v j-jevem poddrevesu,
    e ne upotevamo morebitnih pokritosti med j-jevimi predniki v drevesu. }
  Pokrito: array[1..4 * MaxN + MaxCeilLg2N] of float;
  Sirina, Zacetek: array[1..MaxCeilLg2N + 1] of integer; { po nivojih }
  Globina: integer; { tevilo nivojev }
  { Pravokotnik i ima koordainte x[2 * i - 1], x[2 * i] ipd. za y. }
  x, y: KoordinateT;
  { x[xi[i]] je i-ta najmanja med koordinatami x[1..2 * N].
    Podobno je yi za y-koordinate.  e je xi[i] = j, je xInv[j] = i. }
  xi, xInv, yi: KazaloT;

  { Izpie vsebino drevesa (tabeli Drevo in Pokrito) po nivojih. }
  procedure IzpisiDrevo;
  var i, j: integer;
  begin
    for i := Globina downto 1 do begin
      Write('(');
      for j := 1 to Sirina[i] do begin
        if j > 1 then Write(' ');
        Write(Drevo[Zacetek[i] + j - 1], ':', Pokrito[Zacetek[i] + j - 1]:0:2);
      end; {for j}
      Write(') ');
    end; {for i}
  end; {IzpisiDrevo}
  
  procedure SpremeniDrevo(xOd, xDo, Nivo, i, Delta: integer);
  var L, M, D, j, jPod: integer; Sir: float;
  begin
    { i-to vozlie drevesa na nivoju Nivo pokriva interval L..D.
      Mi pa bi radi v drevesu spremenili interval xOd..xDo. }
    L := ((i - 1) shl (Nivo - 1)) + 1; 
    D := i shl (Nivo - 1); if D >= 2 * N then D := 2 * N - 1;
    j := Zacetek[Nivo] + i - 1; { trenutno vozlie je Drevo[j] }
    Sir := x[xi[D + 1]] - x[xi[L]]; { irina intervala, ki ga predstavlja trenutno vozlie }
    { WriteLn('SpremeniDrevo(', xOd, '..', xDo, ', Delta = ', Delta, 
             '), nivo ', Nivo, ', i = ', i, ', torej ', L, '..', D); }
    if Nivo = 1 then begin Assert(xOd <= L); Assert(L = D); Assert(D <= xDo); end;
    if (xOd <= L) and (xDo >= D) then 
      { xOd..xDo pokriva tudi celoten interval L..D. }
      Inc(Drevo[j], Delta)
    else begin
      { Treba se bo posebej posvetiti levemu in desnemu poddrevesu. }
      Assert(Nivo > 1);
      M := (L + 1 shl (Nivo - 2)) - 1;
      { WriteLn('M = ', M); }
      { Levo poddrevo pokriva interval L..M, desno pa (M + 1)..D. }
      if xOd <= M then SpremeniDrevo(xOd, xDo, Nivo - 1, 2 * i - 1, Delta);
      if xDo > M then SpremeniDrevo(xOd, xDo, Nivo - 1, 2 * i, Delta);
    end; {if}
    { Popravimo vrednost Pokrito[j] (e se je kaj spremenilo). }
    if Drevo[j] > 0 then Pokrito[j] := Sir { j je popolnoma pokrit }
    else if Nivo = 1 then Pokrito[j] := 0  { j je nepokrit list }
    else begin
      { Ker j sam po sebi ni pokrit, ima pa eno ali dve poddrevesi,
        bomo seteli pokritost obeh poddreves. }
      jPod := Zacetek[Nivo - 1] + 2 * i - 2;
      Pokrito[j] := Pokrito[jPod];     { upotevajmo levo poddrevo }
      if jPod + 1 < Zacetek[Nivo] then { imamo tudi desno poddrevo? }
        Pokrito[j] := Pokrito[j] + Pokrito[jPod + 1];
    end; {if}
    { if Nivo = Globina then IzpisiDrevo; }
  end; {SpremeniDrevo}

var
  T: text; StPrimerov, i, p: integer;
  Ploscina: float;
begin {UnijaPravokotnikov}
  if ParamCount >= 1 then Assign(T, ParamStr(1))
  else Assign(T, 'rtk98s2n3.in');
  Reset(T); StPrimerov := 0;
  while true do begin
  
    { Preberimo naslednji testni primer. }
    ReadLn(T, N); if N <= 0 then break;
    for i := 1 to N do 
      ReadLn(T, x[2 * i - 1], y[2 * i - 1], x[2 * i], y[2 * i]);
    
    { Pripravimo kazali koordinat.  Za xi potrebujemo tudi obrnjen 
      indeks -- xInv[i] naj pove, katera po vrsti je koordinata x[i]. 
      xi[i] pa naj pove, katera iz tabele x je i-ta po vrsti. }
    UrediKoordinate(x, xi); 
    for i := 1 to 2 * N do xInv[xi[i]] := i;
    UrediKoordinate(y, yi);
    
    { Pripravimo prazno drevo segmentov. }
    Globina := 1; Sirina[Globina] := 2 * N - 1;
    while Sirina[Globina] > 1 do 
      begin Inc(Globina); Sirina[Globina] := (Sirina[Globina - 1] + 1) div 2; end;
    Zacetek[1] := 1; 
    for i := 2 to Globina do Zacetek[i] := Zacetek[i - 1] + Sirina[i - 1];
    for i := 1 to Zacetek[Globina] + Sirina[Globina] - 1 do 
      begin Drevo[i] := 0; Pokrito[i] := 0; end;
    
    { Pometajmo ravnino z vodoravnim pasom. }
    Ploscina := 0;
    for i := 1 to 2 * N - 1 do begin
      { Gledamo pas med y[yi[i]] in y[yi[i + 1]]. }
      p := (yi[i] + 1) div 2; { tevilka pravokotnika, ki mu pripada ta stranica }
      { e je yi[i] mod 2 = 1, je to spodnja stranica in jo moramo
        dodati v drevo (Delta = 1), sicer pa je zgornja in jo 
        moramo vzeti iz drevesa (Delta = -1). }
      { WriteLn('p = ', p); }
      SpremeniDrevo(xInv[2 * p - 1], xInv[2 * p] - 1, Globina, 1, 2 * (yi[i] mod 2) - 1);
      { Write('[', x[2 * p - 1]:0:2, ', ', x[2 * p]:0:2, ']  '); }
      { Dodajmo ploino, ki jo pokriva unija na tem pasu. }
      Ploscina := Ploscina + Pokrito[Zacetek[Globina]] * (y[yi[i + 1]] - y[yi[i]]);
      { WriteLn('irina intervalov: ', Pokrito[Zacetek[Globina]]:0:2, 
                ', viina pasu: ', (y[yi[i + 1]] - y[yi[i]]):0:2, ' --> ', Ploscina:0:2); }
    end; {for}
    
    Inc(StPrimerov); 
    WriteLn('Test case #', StPrimerov);
    WriteLn('Total explored area: ', Ploscina:0:2);
    WriteLn;
  end; {while}
end. {UnijaPravokotnikov}
