program KdoJeGlavni; { RTK 1999.3.4 }
{ Ta program simulira dogajanje v omreju in izpisuje na zaslon 
  stanje raunalnikov (glavni ali pomoni) in oddaje ter sprejeme
  sporoil.  Uporabnik lahko s tipkami 1..N (N = tevilo 
  raunalnikov) simulira izpad oz. ponovni vklop raunalnika. 
  S tipko O lahko postavlja "ovire" med raunalnike, tako da pridejo
  sporoila, ki jih odpoilja raunalnik i, do raunalnika j
  le v primeru, e med i in j ni nobene ovire.  Tako lahko 
  simuliramo razpad omreja na ve nepovezanih delov. }
uses Crt;

type
  SporociloP = ^SporociloT;
  SporociloT = record
    St: integer;      { vsebina sporoila }
    Cas: integer;     { do kdaj mora prispeti na cilj }
    Nasl: SporociloP; { naslednje sporoilo v vrsti }
  end; {SporociloT}

  RacunalnikP = ^RacunalnikT;
  RacunalnikT = object
    { Ta del objekta so interne zadeve, ki jih uporablja simulator.
      Tekmovaleva koda naj z njimi ne bi imela opravka. }
    StRac, Kdaj: integer;
    Sporocila: SporociloP;
    Stanje: char; { P (pomozni), G (glavni), x (mrtev), ? (iv, a e ni doloeno, ali je P ali G) }
    Prekinitev: boolean;
    constructor Init(StRac_, Kdaj_: integer);
    destructor Done;
    procedure BrisiSporocila;

    { Vmesnik, kot ga obljublja opis naloge. }
    function VrniStRacunalnika: integer;
    procedure PostaviGlavniRac;
    procedure PostaviPodporniRac;
    procedure PosljiVsem(St: integer);
    function Prejmi(var St: integer): boolean;

    { Za preostanek objekta skrbi tekmovalec. }
    SemGlavni: boolean; { trenutno stanje raunalnika
      (vmesnik, kot ga opisuje naloga, namre ne predvideva,
      da bi imel podprogram VsakoSekundo dostop do 
      simulatorjevega polja Stanje ali esa podobnega) }
    MinStPrejSek, { najmanja tevilka, ki smo jo prejeli v prejnji sekundi }
    StSek: integer; { Zacni postavi StSek na 0, VsakoSekundo jo povea za 1 }
    procedure Zacni;
    procedure VsakoSekundo;
  end; {RacunalnikT}

procedure RacunalnikT.Zacni; 
begin
  SemGlavni := false; 
  MinStPrejSek := -1; 
  StSek := 0;
  PostaviPodporniRac; 
end; {RacunalnikT.Zacni}

(*
{ Zgodneja verzija podprograma VsakoSekundo.
  Tu je e ve napak, zaradi katerih se lahko zgodi,
  da se pojavi v omreju istoasno po ve glavnih
  raunalnikov. }
procedure RacunalnikT.VsakoSekundo; 
var 
  PraviGlavni, Glavni, i: integer; 
begin
  if SemGlavni then begin 
    PosljiVsem(VrniStRacunalnika); 
  end else begin 
    i := 0; 
    PraviGlavni := -1; 
    while Prejmi(Glavni) do begin 
      i := i + 1; 
      if (PraviGlavni = -1) or (Glavni < PraviGlavni) then 
        PraviGlavni := Glavni; 
    end; {while}; 
    if i = 0 then begin 
      { Glavnega raunalnika ni; predlagajmo sebe za glavnega. } 
      PosljiVsem(VrniStRacunalnika); 
    end else begin 
      { Konec glasovanja, glavni bo tisti z najmanjo tevilko. } 
      if VrniStRacunalnika = PraviGlavni then begin 
        SemGlavni := true; 
        PostaviGlavniRac; 
      end else begin 
        SemGlavni := false; 
        PostaviPodporniRac; 
      end; {if} 
    end; {if}
  end; {if}
end; {RacunalnikT.VsakoSekundo} 
*)

procedure RacunalnikT.VsakoSekundo; 
var 
  St, MinSt, Glavni: integer; 
begin
  StSek := StSek + 1;
  if SemGlavni then begin 
    { V primeru, ko omreje razpade na ve nepovezanih delov, 
      si lahko vsak del izglasuje svoj glavni raunalnik.  Ko
      se povezave spet vzpostavijo, obstaja zdaj ve glavnih
      raunalnikov, ki vpijejo drug mimo drugega.  Zato morajo
      v takem primeru vsi razen enega utihniti. }
    MinSt := -1;
    while Prejmi(St) do 
      if (MinSt = -1) or (St < MinSt) then MinSt := St;
    if (MinSt > -1) and (MinSt < VrniStRacunalnika) then
      { Eden od drugih glavnih raunalnikov ima manjo tevilko
        kot mi, zato postanimo podporni raunalnik. }
      begin SemGlavni := false; PostaviPodporniRac end
    else
      { Ni drugega glavnega raunalnika oz. vsaj ne takega
        z manjo tevilko od nae.  Zato ostanimo glavni. }
      PosljiVsem(VrniStRacunalnika); 
  end else begin 
    MinSt := -1; 
    while Prejmi(St) do  
      if (MinSt = -1) or (St < MinSt) then MinSt := St;
    Glavni := MinStPrejSek;
    if (Glavni = -1) or ((MinSt > 0) and (MinSt < Glavni)) then Glavni := MinSt;
    
    { Poglejmo zdaj, kako je z glasovanjem. }
    if Glavni = -1 then begin 
      { Glavnega raunalnika ni; predlagajmo sebe za glavnega. 
        e smo se ravnokar zbudili (StSek = 1), tega raje ne
        storimo, ker se pa lahko po nakljuju zgodi, da v 
        tisti <= 1 sekundi, kolikor dolgo smo budni, ne
        dobimo nobenega sporoila, eprav v omreju v resnici
        obstaja glavni raunalnik.  V tem primeru bi delali
        zgolj zmedo, e bi zdaj zaeli vpiti svojo tevilko. }
      if StSek >= 2 then PosljiVsem(VrniStRacunalnika); 
    end else begin 
      { Glasovanje je v teku, glavni bo tisti z najmanjo tevilko. 
        e smo to mi, se spodobi, da poljemo takoj svojo tevilko,
        drugae bomo to storili ele ez eno sekundo in 
        se bodo drugi medtem e pregovarjali.  Preden pa se 
        dokonno razglasimo za glavnega, bi radi, da bi bila
        naa tevilka najmanja dve sekundi zaporedoma.
        To je potrebno zaradi monosti, da se naa sporoila
        na poti zadrijo eno sekundo in smo lahko zato ele
        po dveh sekundah prepriani, da ni e kje kak drug
        raunalnik, ki za nao zmago ne ve in e kar vpije
        svojo tevilko. } 
      if VrniStRacunalnika = Glavni then begin 
        if MinStPrejSek = Glavni then 
          begin SemGlavni := true; PostaviGlavniRac; end;
        PosljiVsem(VrniStRacunalnika);
      end else begin 
        SemGlavni := false; 
        PostaviPodporniRac; 
      end; {if} 
    end; {if}
    MinStPrejSek := MinSt;
  end; {if}
end; {RacunalnikT.VsakoSekundo} 

{-----------------------------------------------------------------------------}
{ Preostanek programa simulira dogajanje v omreju in klie                   }
{ tekmovalevi funkciji Zacni in VsakoSekundo.                                }
{-----------------------------------------------------------------------------}

const N = 8; { tevilo raunalnikov }
var
  Racunalniki: array[1..N] of RacunalnikP;
  Cas: integer;

constructor RacunalnikT.Init(StRac_, Kdaj_: integer);
begin
  StRac := StRac_; Kdaj := Kdaj_; 
  Sporocila := nil;
  Stanje := '?';
  Prekinitev := false;
end; {RacunalnikT.Init}

procedure RacunalnikT.BrisiSporocila;
var P: SporociloP;
begin
  while Sporocila <> nil do begin
    P := Sporocila^.Nasl;
    Dispose(Sporocila);
    Sporocila := P;
  end; {while}
end; {RacunalnikT.BrisiSporocila}

destructor RacunalnikT.Done;
  begin BrisiSporocila; end;

function RacunalnikT.VrniStRacunalnika: integer;
  begin VrniStRacunalnika := StRac; end;

procedure RacunalnikT.PostaviGlavniRac;
  begin Stanje := 'G'; end;
  
procedure RacunalnikT.PostaviPodporniRac;
  begin Stanje := 'P'; end;
  
procedure RacunalnikT.PosljiVsem(St: integer);
var i: integer; P: SporociloP;
begin
  WriteLn(Cas:10, ': #', StRac, ' polje ', St);
  { Naloga zagotavlja, da sporoila prispejo na cilj
    v manj kot eni sekundi.  Zato si za vsak raunalnik
    izmislimo nakljuen as (od 1 do 999 ms), v katerem
    bo sporoilo prilo do njega. }
  i := St;
  while i > 1 do 
    if Racunalniki[i - 1]^.Prekinitev then break else i := i - 1;
  while i <= N do begin
    New(P);
    P^.Nasl := Racunalniki[i]^.Sporocila;
    P^.St := St;
    P^.Cas := Cas + Random(999) + 1;
    Racunalniki[i]^.Sporocila := P;
    if Racunalniki[i]^.Prekinitev then break else i := i + 1;
  end; {for i}
end; {RacunalnikT.PosljiVsem}
  
function RacunalnikT.Prejmi(var St: integer): boolean;
var P, Prej: SporociloP;
begin
  Prej := nil; P := Sporocila;
  { Iz vrste vzemimo poljubno sporoilo, ki naj bi
    ta raunalnk doseglo do trenutnega asa. }
  while P <> nil do begin
    if P^.Cas <= Cas then begin
      Prejmi := true; St := P^.St;
      if Prej <> nil then Prej^.Nasl := P^.Nasl
      else Sporocila := P^.Nasl;
      WriteLn(Cas:10, ': #', StRac, ' prevzame ', St);
      Dispose(P); exit;
    end; {if}
    Prej := P; P := P^.Nasl;
  end; {while}
  Prejmi := false;
end; {RacunalnikT.Prejmi}

procedure PrikaziStanje;
var i, StGlavnih: integer;
begin
  Write(Cas:10, ': ', '                  stanje zdaj ');
  StGlavnih := 0;
  for i := 1 to N do with Racunalniki[i]^ do begin
    Write(Stanje);
    if Stanje = 'G' then StGlavnih := StGlavnih + 1;
    if i < N then 
      if Racunalniki[i]^.Prekinitev then Write('|') else Write(' ');
  end; {for}
  Write('  ');
  { Na desni napiimo 0, e ni glavnega raunalnika,
    in znake ##, e sta glavna dva ali ve.  
    To slednje naj se ne bi moglo zgoditi, razen
    e omreje razpade na ve nepovezanih delov ali pa 
    e ni v naem protokolu e kakna neodkrita napaka. }
  if StGlavnih = 0 then Write('0')
  else if StGlavnih > 1 then begin
    Write('#'); while StGlavnih > 1 do 
      begin Write('#'); StGlavnih := StGlavnih - 1; end;
  end; {if}
  WriteLn;
end; {PrikaziStanje}

var Sek, Ms, Nasl, i, k: integer; c: char;
begin {KdoJeGlavni}
  WriteLn('Po vsakem koraku simulacije lahko s tipkami 1..', N, ' priiga');
  WriteLn('in izklaplja raunalnike.  Q = konec, P = pavza.');
  WriteLn('S pritiskom na O lahko postavlja med raunalniki ovire, ki');
  WriteLn('blokirajo irjenje sporoil (kot da omreje razpade na ve delov).');
  
  for i := 1 to N do 
    Racunalniki[i] := New(RacunalnikP, Init(i, 
      Random(999 div N) * N + i - 1));
  for i := 1 to N do Racunalniki[i]^.Zacni;
      
  Sek := 0;
  while true do begin
    Ms := 0; 
    for k := 1 to N do begin
    
      { Kateri je naslednji raunalnik, na katerem je treba
        izvesti podprogram VsakoSekundo?  Na vsakem se izvaja
        ta podprogram v presledkih po eno sekundo, vendar
        ne na vseh naenkrat, pa pa pri enem nekaj milisekund
        prej, pri drugem nekaj kasneje.  V ta namen imamo polje Kdaj. 
        Vrednosti Kdaj smo izbrali tako, da imajo vsi raunalniki
        razline (ker imajo vsi razline ostanke po modulu N). }
      Nasl := 0;
      for i := 1 to N do if Racunalniki[i]^.Kdaj > Ms then
        if Nasl = 0 then Nasl := i
        else if Racunalniki[i]^.Kdaj < Racunalniki[Nasl]^.Kdaj then Nasl := i;
        
      with Racunalniki[Nasl]^ do begin
        Delay(1000 * Sek + Kdaj - Cas);
        Ms := Kdaj; Cas := 1000 * Sek + Ms;
        { Mrtev raunalnik izgublja sporoila. }
        if Stanje = 'x' then BrisiSporocila
        else VsakoSekundo;
      end; {with}
      PrikaziStanje;

      { Poglejmo, e je uporabnik pritisnil na kakno tipko. }
      while KeyPressed do begin
        c := UpCase(ReadKey);
        if c = 'Q' then break;
        if (c = 'P') or (c = ' ') then 
          begin Write('(pavza)', #13); c := UpCase(ReadKey); end;
        if c = 'O' then begin
          WriteLn('Pritisni tevilko raunalnika, kjer bi postavil ali umaknil oviro.');
          c := UpCase(ReadKey);
          i := Ord(c) - Ord('0');
          if (i < 1) or (i > N) then break;
          Racunalniki[i]^.Prekinitev := not Racunalniki[i]^.Prekinitev;
        end else begin
          { Tipke '1', '2', itd. povzroijo vklop/izklop raunalnika. }
          i := Ord(c) - Ord('0');
          if (i < 1) or (i > N) then break;
          with Racunalniki[i]^ do
            if Stanje = 'x' then begin Stanje := '?'; Zacni; end
            else Stanje := 'x';
        end; {if}
        PrikaziStanje;
      end; {while}
      
      if c = 'Q' then break;
    end; {for k}
    Sek := Sek + 1;    
    if c = 'Q' then break;
  end; {while}
  
  for i := 1 to N do Dispose(Racunalniki[i], Done);
end. {KdoJeGlavni}
