program PrestopneSekunde; { RTK 2000.1.1 }
type DatumT = packed array[1..10] of char; 

  { Reevalci na tekmovanju so lahko predpostavili, 
    da je ta podprogram e dan.  Vrednost, ki jo izrauna,
    se imenuje "mean Julian date" in teje dneve od zaetka
    17. novembra 1858.  Glej npr. http://tycho.usno.navy.mil/mjd.html . }
  function MJD(Datum: DatumT): integer; 
  const 
    Stevke = ['0'..'9'];
    Meseci: array[1..12] of integer = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
    
    function C(ch: char): integer; 
      begin C := Ord(ch) - Ord('0'); end;
      
  var D, M, L, LL, n, i: integer;
  begin
    { Preverimo, e je niz prave oblike. }
    if not ((Datum[1] in Stevke) and (Datum[2] in Stevke)
      and (Datum[3] in Stevke) and (Datum[4] in Stevke)
    and (Datum[5] = '-') and (Datum[6] in Stevke)
    and (Datum[7] in Stevke) and (Datum[8] = '-')
    and (Datum[9] in Stevke) and (Datum[10] in Stevke)) 
    then begin WriteLn('Niz "', Datum, '" ni oblike "yyyy-mm-dd".'); Halt; end;
    { Izluimo dan, mesec in leto. }
    L := 1000 * C(Datum[1]) + 100 * C(Datum[2]) + 10 * C(Datum[3]) + C(Datum[4]);
    M := 10 * C(Datum[6]) + C(Datum[7]);
    D := 10 * C(Datum[9]) + C(Datum[10]);
    { Izraunajmo tevilo dni od zaetka leta 1601 do zaetka leta L. 
      Tako nam ne bo treba komplicirati z razliko med julijanskim
      in gregorijanskim koledarjem. }
    LL := L - 1601; n := (LL div 400) * (365 * 400 + 97); { na vsakih 400 let pride 97 prestopnih }
    LL := LL mod 400; n := n + (LL div 100) * (365 * 100 + 24); { na vsakih 100 let pride 24 prestopnih }
    LL := LL mod 100; n := n + 365 * LL + (LL div 4); { na vsaka 4 leta pride eno prestopno }
    { Dodajmo dneve v mesecih pred M. }
    for i := 1 to M - 1 do n := n + Meseci[i];
    if (M >= 3) and (L mod 4 = 0) then { Dodajmo 29. februar. }
      if (L mod 400 = 0) or not (L mod 100 = 0) then n := n + 1;
    n := n + D;
    { Zdaj je n tevilo dni med zaetkom leta 1601 in 
      zaetkom dneva D. M. L.  Pravi MJD (modified Julian date)
      pa teje od zaetka 17. novembra 1858; zanj bi dobili n = 94188. }
    MJD := n - 94188;
  end; {MJD}

var 
  Prestopne: text; 
  Rojstvo, Danes, Kdaj: DatumT; 
  Starost, PrestopnihSek: integer; 
  PrestopnihSekObRojstvu, PrestopnihSekDanes: integer; 

begin {PrestopneSekunde}
  PrestopnihSekObRojstvu := 0; PrestopnihSekDanes := 0; 
  if ParamCount >= 1 then 
    begin Rojstvo := ParamStr(1); WriteLn('Rojstvo: ', Rojstvo); end
  else begin Write('Rojstvo: '); ReadLn(Rojstvo); end;
  if ParamCount >= 2 then 
    begin Danes := ParamStr(2); WriteLn('Danes: ', Danes); end
  else begin Write('Danes: '); ReadLn(Danes); end;
  
  Assign(Prestopne, 'prestopne.txt'); Reset(Prestopne);
  while not Eof(Prestopne) do begin 
    ReadLn(Prestopne,  Kdaj, PrestopnihSek); 
    if MJD(Kdaj) <= MJD(Rojstvo) then 
      PrestopnihSekObRojstvu := PrestopnihSek; 
    if MJD(Kdaj) <= MJD(Danes) then 
      PrestopnihSekDanes := PrestopnihSek; 
  end; {while}
  Close(Prestopne);
  
  Starost := (MJD(Danes) - MJD(Rojstvo)) * 24 * 60 * 60 + 
             (PrestopnihSekDanes - PrestopnihSekObRojstvu); 
  WriteLn('Od rojstnega do dananjega datuma (ob polnoi) je minilo ', 
          Starost, ' sekund.'); 
end. {PrestopneSekunde} 
    