program PitagorejskeTrojice2; { RTK 2001.1.4 }

{-----------------------------------------------------------------------------}

{ Ta reitev si pomaga z nekim izrekom iz teorije tevil:
  e poiemo vse z-jeve prafaktorje, ki imajo obliko 4k + 1,
  in so njihove stopnje recimo d_1, ..., d_r,
  je mogoe z^2 kot vsoto kvadratov dveh pozitivnih celih
  tevil zapisati na (2 d_1 + 1)(2 d_2 + 1)...(2 d_r + 1) - 1
  nainov.  (Pri tem npr. 3^2 + 4^2 in 4^2 + 3^2 tejeta
  kot dva razlina zapisa vrednosti 5^2 z dvema kvadratoma.) 
  - http://mathworld.wolfram.com/PythagoreanTriple.html 
  - The On-line Encyclopedia of Integer Sequences, A046080,
    http://www.research.att.com/projects/OEIS?Anum=A046080
  - A. H. Beiler: "Recreations in the Theory of Numbers",
    Dover, 1966, str. 116--117, 140--142.
  - G. H. Hardy, E. M. Wright: "An Introduction to the 
    Theory of Numbers", 5. izd., Oxford, 1980, razd. 16.9--16.10
    (str. 241--243). 
  Za razcep z-ja na prafaktorje ga bomo kar poskuali deliti
  z vedno vejimi pratevili.  Seznam pratevil si pripravimo
  na zaetku (z Eratostenovim reetom) in ga potem uporabljamo
  za vse z-je, ki nas zanimajo. }
const 
  MaxSqrtB = 65536; 
  {MaxPrast = 6543; -- toliko je res pratevil, vendar potrebujemo
   zaradi podprograma PsevdoReseto vejo tabelo 'pratevil' }
  MaxPrast = 21846; { 3 + (65536 - 7) / 3 }
var 
  Prast: array[1..MaxPrast] of integer;
  nPrast: integer;

  { Poie vsa pratevila, manja od m. }
  procedure Eratosten(m: integer);
  var 
    Reseto: packed array[0..MaxSqrtB - 1] of boolean;
    i, j: integer;
  begin
    for i := 2 to m do Reseto[i] := Odd(i);
    nPrast := 1; Prast[nPrast] := 2; 
    for i := 3 to m do begin
      if Reseto[i] then begin
        Inc(nPrast); Prast[nPrast] := i;
        j := i * i; while j <= m do 
          begin Reseto[j] := false; Inc(j, 2 * i); end;
      end; {if}
    end; {for}
  end; {Eratosten}
  
  { Poie vsa pratevila, manja od m, med njimi pa pusti tudi
    precej sestavljenih tevil (izloi le vekratnike 2 in 3). }
  procedure PsevdoReseto(m: integer);
  var p: integer;
  begin
    Prast[1] := 2; Prast[2] := 3; Prast[3] := 5;
    nPrast := 3; p := 7;
    while p <= m do begin
      nPrast := nPrast + 1; 
      Prast[nPrast] := p;
      if Odd(nPrast) then p := p + 2
      else p := p + 4;
    end; {while}
  end; {PsevdoReseto}

  { Faktorizira tevilo z in vrne tevilo trojic s hipotenuzo z. 
    e je UporabiPrastevila = false, bo zaporedje deliteljev
    generiral sam (poleg 2 in 3 bo vzel e vsa liha tevila, ki
    niso vekratniki 3). }
  function StTrojic(z: integer; UporabiPrastevila: boolean): integer;
  var zz, nz, i, p, d, StPrast: integer;
  begin
    { Koliko je pratevil, manjih ali enakih Sqrt(b)? }
    if UporabiPrastevila then StPrast := nPrast
    else begin StPrast := 3 + (Trunc(Sqrt(z)) - 4) div 3; 
               Prast[1] := 2; Prast[2] := 3; Prast[3] := 5 end;
    zz := z; nz := 1; { nz = tevilo trojic pri tem z }
    { Razcepimo z na prafaktorje. }
    for i := 1 to StPrast do begin
      if UporabiPrastevila or (i <= 3) then p := Prast[i]
      else if Odd(i) then p := p + 4 else p := p + 2;
      if p * p > zz then break;
      { Poskusimo z deliti s p. }
      d := 0; while zz mod p = 0 do 
        begin zz := zz div p; d := d + 1; end;
      { Prafaktor p se v razcepu tevila z pojavlja s stopnjo d.
        e je p oblike 4k + 1, moramo nz pomnoiti z 2d + 1. }
      if (d > 0) and (p mod 4 = 1) then 
        nz := nz * (2 * d + 1);
    end; {for i}
    { Ostal je e prafaktor zz s stopnjo 1. }
    if (zz > 1) and (zz mod 4 = 1) then nz := nz * (2 * 1 + 1);
    { Pravo tevilo trojic je nz - 1. }
    StTrojic := nz - 1;
  end; {StTrojic}
  
  function MnozicnaFaktorizacija(a, b: integer; UporabiPrastevila: boolean): integer;
  const C = 1; { izkae se, da je korist od vejega C (npr. C = 10) zanemarljiva }
  var 
    zz, nz: array[0..C * MaxSqrtB - 1] of integer;
    zOd, zDo, zn, zd, i, p, MaxZD, d, t, n, StPrast: integer;
  begin
    { Katero je najveje pratevilo, manje ali enako Sqrt(b)? }
    if UporabiPrastevila then MaxZD := Prast[nPrast]
    else MaxZD := Trunc(Sqrt(b));
    { Koliko je pratevil, manjih ali enakih Sqrt(b)? }
    if UporabiPrastevila then StPrast := nPrast
    else begin StPrast := 3 + (Trunc(Sqrt(b)) - 4) div 3;
               Prast[1] := 2; Prast[2] := 3; Prast[3] := 5; end;
    MaxZD := C * MaxZD; { Faktorizirali bomo toliko z-jev naenkrat. }
    { Lotimo se faktorizacije. }
    zOd := a; n := 0;
    while zOd <= b do begin
      zDo := zOd + MaxZD - 1; if zDo > b then zDo := b;
      zn := zDo - zOd + 1;
      { Faktorizirali bomo zn tevil, namre zOd..zDo. }
      for zd := 0 to zn - 1 do 
        begin zz[zd] := zOd + zd; nz[zd] := 1 end;
      { Preglejmo vsa pratevila, manja ali enaka Sqrt(b). }
      for i := 1 to StPrast do begin
        if UporabiPrastevila or (i <= 3) then p := Prast[i]
        else if Odd(i) then p := p + 4 else p := p + 2;
        { Preglejmo vse vekratnike p-ja na intervalu zOd..zDo. }
        zd := ((zOd + p - 1) div p) * p - zOd;
        while zd < zn do begin 
          { tevilo zOd + zd je vekratnik p (razen e niso v tabeli 
            Prast tudi kakna sestavljena tevlia).  Poglejmo 
            stopnjo p-ja v razcepu tevila (zOd * zd) na prafaktorje. }
          t := zz[zd]; {if UporabiPrastevila then Assert((t mod p) = 0);}
          d := 0; while t mod p = 0 do begin t := t div p; d := d + 1 end;
          zz[zd] := t;
          { e je p oblike 4k + 1, prispeva k tevilu trojic. }
          if (d > 0) and (p mod 4 = 1) then nz[zd] := nz[zd] * (2 * d + 1);
          zd := zd + p; { Pojdimo na naslednji vekratnik p. }
        end; {while}
      end; {for i}
      { Zdaj je od vsakega z ostal e najve en prafaktor in to s stopnjo 1. }
      for zd := 0 to zn - 1 do begin
        if (zz[zd] > 1) and (zz[zd] mod 4 = 1) then 
          nz[zd] := nz[zd] * (2 * 1 + 1);
        n := n + nz[zd] - 1; { Spotoma e setevajmo tevilo vseh trojic. }
      end; {for zd}
      zOd := zDo + 1;
    end; {while}
    MnozicnaFaktorizacija := n;
  end; {MnozicnaFaktorizacija}

{-----------------------------------------------------------------------------}

{ e en koristen izrek iz teorije tevil: e pregledamo vse 
  pare (m, n), za katere velja: 
    (1) n < m; 
    (2) m in n nista oba soda ali oba liha; 
    (3) m in n sta si tuja,
  bomo po formuli 
    (x = m^2 - n^2, y = 2mn, z = m^2 + n^2) 
  dobili ravno vse take primitivne pitagorejske trojice,
  pri katerih je x sod in y lih. 
    Glej npr. Joe Grasselli, "Diofantske enabe", 1984, razdelek 10.
    Ker nas zanimajo tudi ostale pitagorejske trojice, 
  bomo vsako dobljeno trojico teli po vekrat -- po dvakrat
  zaradi (y, x, z) in e po (floor(b/z) - ceil(a/z) + 1)-krat,
  da zajamemo tudi neprimitivne trojice. }

  { Po znanem Evklidovem algoritmu izrauna najveji
    skupni delitelj nenegativnih celih tevil a in b. }
  function gcd(a, b: integer): integer;
  begin
    while (a > 0) and (b > 0) do 
      if a < b then b := b mod a
      else a := a mod b;
    gcd := a + b;
  end; {gcd}

  { Vrne tevilo pitagorejskih trojic s hipotenuzo z,
    za katere je a <= z <= b.  Pri tem si pomaga z zgoraj
    omenjenim izrekom. }
  function DveZanki(a, b: integer): integer;
  var k, l, n, z, MaxK: integer;
  begin
    MaxK := Trunc(Sqrt(b)); n := 0;
    for k := 2 to MaxK do begin
      if Odd(k) then l := 2 else l := 1;
      while l < k do begin
        if gcd(k, l) = 1 then begin { Sta si k in l tuja? }
          z := k * k + l * l;
          if z > b then break; { l-ji od tu naprej so za ta k e preveliki. }
          n := n + 2 * (b div z - (a + z - 1) div z + 1);
        end; {if}
        l := l + 2;
      end; {while l < k}
    end; {for k}
    DveZanki := n;
  end; {DveZanki}

{-----------------------------------------------------------------------------}
  
var 
  n, z, a, b, Ponovitev, p: integer; 
  c: char; 
  ImamoPrastevila: boolean; 
begin {PitagorejskeTrojice2}

  if ParamCount >= 2 then begin
    Val(ParamStr(1), a, n); Val(ParamStr(2), b, n);
  end else begin
    WriteLn('Sintaksa: rtk01s1n4 a b [R [A [StPonovitev]]');
    WriteLn('R = algoritem za iskanje pratevil.  Mone vrednosti:');
    WriteLn('    E = Eratostenovo reeto (default)');
    WriteLn('    P = PsevdoReeto (v tabelo vpie tevila, ki niso vekratniki 2 ali 3)');
    WriteLn('    - = ni -- podprogram za faktorizacijo si sproti rauna delitelje,');
    WriteLn('        in sicer iste, kot bi jih dobil tudi od podprograma PsevdoReseto');
    WriteLn('A = algoritem za tetje trojic.  Mone vrednosti:');
    WriteLn('    0 = faktorizacija vsakega z posebej (default)');
    WriteLn('    1 = faktorizacija po ve zaporednih z-jev naenkrat;');
    WriteLn('        potrebuje e O(min(z, Sqrt(b)) pomnilnika');
    WriteLn('    2 = generiranje primitivnih pitagorejskih trojic');
    WriteLn('        z dvema zankama do Sqrt(b)');
    WriteLn('StPonovitev = program tolikokrat ponovi reetanje in tetje trojic.');
    WriteLn('              Koristno za natanneje merjenje asa.  Default: 1.');
    exit;
  end; {if}
  
  if ParamCount >= 5 then Val(ParamStr(5), Ponovitev, n)
  else Ponovitev := 1;
  
  for p := 1 to Ponovitev do begin
  
    { Poiimo pratevila, ki jih bomo potrebovali 
      pri razcepu tevil a..b na prafaktorje. }
    if ParamCount >= 3 then c := UpCase(ParamStr(3)[1])
    else c := 'E';
    ImamoPrastevila := true;
    case c of
    
      'E': begin
        if p = 1 then Write('Eratostenovo reeto, ');
        Eratosten(Trunc(Sqrt(b)));
      end; {Eratosten}
      
      'P': begin
        if p = 1 then Write('PsevdoReseto, ');
        PsevdoReseto(Trunc(Sqrt(b)));
      end; {PsevdoReseto}
      
      else begin
        if p = 1 then Write('Brez seznama deliteljev, ');
        ImamoPrastevila := false;
      end; {brez seznama deliteljev}
    end; {case c}

    if ParamCount >= 4 then c := UpCase(ParamStr(4)[1])
    else c := '0';
    case c of

      '0': begin
        if p = 1 then Write('posamina faktorizacija, ');
        n := 0; { skupno tevilo trojic }
        for z := a to b do n := n + StTrojic(z, ImamoPrastevila);
      end; {posamina faktorizacija}

      '1', 'M': begin
        if p = 1 then Write('mnoina faktorizacija, ');
        n := MnozicnaFaktorizacija(a, b, ImamoPrastevila);
      end; {mnoina faktorizacija}
      
      '2', 'Z': begin
        if p = 1 then Write('generiranje primitivnih trojic, ');
        n := DveZanki(a, b);
      end; {dve zanki}
    end; {case c}
    
  end; {for p}

  WriteLn;
  WriteLn('StPonovitev = ', Ponovitev, ', a = ', a, ', b = ', b, 
          ', n = ', b - a + 1, ', t. trojic = ', n, '.');
end. {PitagorejskeTrojice2}
